/*------------------------------------------------------------------------------
*
* Copyright 2001-2015 Dialogic(R) Corporation
*
* All Rights Reserved
*
* This software is the property of Dialogic Corporation and/or its
* subsidiaries ("Dialogic"). This copyright notice may not be removed,
* modified or obliterated without the prior written permission of
* Dialogic.
*
* No right, title, ownership or other interest in the software is hereby
* granted or transferred. The information contained herein is subject
* to change without notice and should not be construed as a commitment of
* Dialogic.
*
* The code is provided by Dialogic "AS IS", without warranty of any
* kind, either expressed or implied. Dialogic shall not be liable for
* any damages arising out of your use of this application, even if it has
* been advised of the possibility of such damages.
*
*------------------------------------------------------------------------------
 *
 * File name:   dssdk.h
 * Project:     Dialogic(R) Diva(R) SDK
 * Description: Main public header file for Diva(R) API
 *
 * Version:     
 *
------------------------------------------------------------------------------*/
#ifndef _DIVA_SERVER_SDK_H_
#define _DIVA_SERVER_SDK_H_


#ifdef __cplusplus
extern "C" {
#endif



#ifdef SDK_DLL
    #ifdef LIB_DYN_EXPORTS
        // for DLL implementation:
        #define DIVA_SERVER_API __declspec(dllexport)
    #else
        // for DLL usage:
        #define DIVA_SERVER_API __declspec(dllimport)
    #endif
#else
    // for static implementation and usage:
    #define DIVA_SERVER_API
#endif



/*------------------------------------------------------------------------------
 *
 * Data types
 *
 -----------------------------------------------------------------------------*/

#ifndef NULL
#define NULL           ((void*)0)
#endif

#ifndef TRUE
#define TRUE           1
#define FALSE          0
typedef int            BOOL;
typedef void*          PVOID;
typedef PVOID          HANDLE;
typedef unsigned char  BYTE;
typedef unsigned short WORD;
typedef unsigned int   DWORD;
typedef unsigned int   UINT;
#endif

typedef HANDLE DivaAppHandle;
typedef HANDLE DivaCallHandle;
typedef HANDLE AppCallHandle;
typedef HANDLE DivaMonitorHandle;
typedef HANDLE AppMonitorHandle;
typedef HANDLE DivaHandle;


/*------------------------------------------------------------------------------
 *
 * Defines
 *
------------------------------------------------------------------------------*/

#define LINEDEV_ALL              0x00000000 // all lines
#define MAX_FAX_SPEED            0x00000000 // maximum speed for fax transmission
#define MAX_MODEM_SPEED          0x00000000 // automatic negotiation
#define MAX_ADDR_LEN             128
#define MAX_SUBADDR_LEN          128
#define FAX_PARAM_LENGTH         21
#define X25_ADDRESS_LENGTH       16
#define IP_ADDRESS_LENGTH        40
#define INVALID_APP_CALL_HANDLE  ((AppCallHandle)-1) // invalid AppCallHandle


/*------------------------------------------------------------------------------
 *
 * SDK specific types
 *
 -----------------------------------------------------------------------------*/

typedef enum
{
    DivaSuccess = 0,
    DivaErrorLineDevice,
    DivaErrorInvalidFunction,
    DivaErrorInvalidHandle,
    DivaErrorInvalidParameter,
    DivaErrorInvalidState,
    DivaErrorOutOfMemory,
    DivaErrorNoCapi,                // Obsolete, now DivaErrorNoDevice
    DivaErrorCapiError,
    DivaErrorDestBusy,
    DivaErrorNoAnswer,
    DivaErrorNoChannel,
    DivaErrorOpenFile,
    DivaErrorUnsupportedFormat,
    DivaErrorReadFile,
    DivaErrorAnotherAppGotThatCall,
    DivaErrorTimeout,
    DivaErrorUnallocatedNumber,
    DivaErrorNotSupported,
    DivaErrorUnspecific,
    DivaErrorReadOnlyParameter,
    DivaErrorDataSize,
    DivaErrorWriteOnlyParameter,
    DivaErrorAlreadyAssigned,
    DivaErrorNoDataAvailable,
    DivaErrorEndOfData,
    DivaErrorFormatNotEnabled,
    DivaErrorNoDevice,
    DivaErrorInsufficientBuffer,
    DivaErrorRejected,
    DivaErrorNoResources,
} DivaReturnCodes;

typedef enum
{
    DivaEventModeNone               = 0x02,     // future use
    DivaEventModeCallback           = 0x04,
    DivaEventModeCallbackEx         = 0x20,
    DivaEventModeCallbackSignal     = 0x40,

    DivaEventModeStdCallback        = 0x80000004,
    DivaEventModeStdCallbackEx      = 0x80000020,
    DivaEventModeStdCallbackSignal  = 0x80000040,
} DivaEventModes;

typedef enum
{
    DivaCallStateIdle = 0,
    DivaCallStateDialing = 1,
    DivaCallStateRinging,
    DivaCallStateOffering,
    DivaCallStateAnswered,
    DivaCallStateProceeding,
    DivaCallStateConnected,
    DivaCallStateOnHold,
    DivaCallStateDisconnecting,
    DivaCallStateDisconnected,
} DivaCallState;

typedef enum
{
    DivaListenNone          = 0x00000000,
    DivaListenAll           = 0xFFFFFFFF,
    DivaListenAllSMS        = 0x00010012,
    DivaListenAllVoice      = 0x00010032,
    DivaListenSpeech        = 0x00000002,
    DivaListenAudio3_1KHz   = 0x00000010,
    DivaListenAudio7KHz     = 0x00000020,
    DivaListenTelephony     = 0x00010000,
    DivaListenFaxG3         = 0x00020000
} DivaListenType;

typedef enum
{
    DivaSignalledCallTypeUnknown = 0,
    DivaSignalledCallTypeAnalog,
    DivaSignalledCallTypeDigital,
    DivaSignalledCallTypeGSM,
    DivaSignalledCallTypeFax
} DivaSignalledCallType;

typedef enum
{
    DivaSignalServiceUnknown        = 0,
    DivaSignalServiceSpeech         = 1,
    DivaSignalServiceDigital        = 2,
    DivaSignalServiceV110           = 8,
    DivaSignalServiceAudio3_1KHz    = 4,
    DivaSignalServiceAudio7KHz      = 5,
    DivaSignalServiceTelefony       = 16,
    DivaSignalServiceFaxG3          = 17,
    DivaSignalServiceFaxG4          = 18,
    DivaSignalServiceVideo          = 6,
} DivaSignalServices;

typedef enum
{
    DivaCallTypeVoice,
    DivaCallTypeFax,
    DivaCallTypeModem,
    DivaCallTypeDigitalData,
    DivaCallTypeX75,
    DivaCallTypeV120,
    DivaCallTypeGSM,
    DivaCallTypeVoIP,
    DivaCallTypeX25,
    DivaCallTypeSMS,
    DivaCallTypeRTPGwMode,
    DivaCallTypeAutoDetect = 100,   // Only for incoming calls.
    DivaCallTypeSignalingOnly = 200,
} DivaCallType;

typedef enum
{   // signalling disconnect codes:
    DivaDRNormalCallClearing = 0,
    DivaDRActiveDisconnect,
    DivaDRBusy,
    DivaDRReject,
    DivaDRNoAnswer,
    DivaDRAlertingNoAnswer,
    DivaDRNumberUnknown,
    DivaDRInvalidNumber,
    DivaDRNumberChanged,
    DivaDRNoChannelAvailable,
    DivaDRCableError,
    DivaDRGeneralNetworkError,
    DivaDRUnspecifiedError,
    DivaDRUnallocatedNumber,
    DivaDRAnotherAppGotThatCall,
    DivaDRDataChannelFailed,
    // fax related disconnect codes:
    DivaDRNoFaxDevice,
    DivaDRFaxTrainingFailed,
    DivaDRFaxRemoteAbort,
    DivaDRFaxLocalAbort,
    // modem related disconnect codes:
    DivaDRModemNegotiationFailed,
    DivaDRModemNoAnswer,
    DivaDRModemCarrierLost,

    // new with 3.0
    DivaDRIncompatibleDestination,
    DivaDRFileAccess,
    DivaDRLowMemory,
    DivaDRIllegalData,

    DivaDRConnectTimeout,

    DivaDRFaxSecureNotSupported,
    DivaDRFaxPartialErrors,
    
} DivaDisconnectReasons;

typedef enum
{
    DivaActiveDiscReasonBusy = 1,
    DivaActiveDiscReasonReject,
    DivaActiveDiscReasonNoAnswer,
    DivaActiveDiscReasonNumberUnknown,
    DivaActiveDiscReasonInvalidNumber,
    DivaActiveDiscReasonNumberChanged,
    DivaActiveDiscReasonUnallocatedNumber,
    DivaActiveDiscReasonOutOfOrder,
} DivaActiveDiscReasons;

typedef enum
{
    DivaRRNormalCallClearing = 0,
    DivaRRUserBusy = 3,
    DivaRRChannelNotAvailable,
    DivaRRFacilityRejected,
    DivaRRChannelNotAccepted,
    DivaRRIncompatibleDestination,
    DivaRRDestinationOutOfOrder,
    DivaRRUserDefined = 0x3400,
} DivaRejectReason;

typedef enum
{
    DivaRedirectReasonUnknown = 0,
    DivaRedirectReasonBusy = 1,
    DivaRedirectReasonNoReply = 2,
    DivaRedirectReasonCallDeflection = 4,
    DivaRedirectReasonDTEOutOfOrder = 9,
    DivaRedirectReasonByCalledDTE = 10,
    DivaRedirectReasonUnconditional = 15,
    DivaRedirectReasonUnavailable = 256,
    DivaRedirectReasonTimeOfDay,
    DivaRedirectReasonDoNotDisturb,
    DivaRedirectReasonFollowMe,
    DivaRedirectReasonAway,
} DivaRedirectReason;

typedef enum
{   
    DivaTRRUnknown = 0,
    DivaTRRBadRequest,
    DivaTRRUnauthorized,
    DivaTRRNotAcceptable,

    DivaTRRDefault = DivaTRRNotAcceptable,
    DivaTRRUserDefined = 0x80000000,
} DivaTransferRejectReason;

typedef enum
{
    DivaRecordEndReasonUndefined = 0,
    DivaRecordEndReasonDisconnected = 1,
    DivaRecordEndReasonTimeout = 2,
    DivaRecordEndReasonSilence = 3,
    DivaRecordEndReasonMaxDTMF = 4,
    DivaRecordEndReasonTerminationDigit = 5,
    DivaRecordEndReasonInterDigitTimeout = 6,
    DivaRecordEndReasonInitialDigitTimeout = 7,
} DivaRecordEndReasons;

typedef enum
{
    DivaSendVoiceEndReasonUnspecified = 0,
    DivaSendVoiceEndReasonDisconnected,
    DivaSendVoiceEndReasonMaxDTMF,
    DivaSendVoiceEndReasonTerminationDigit,
    DivaSendVoiceEndReasonInterDigitTimeout,
    DivaSendVoiceEndReasonInitialDigitTimeout,
    DivaSendVoiceEndReasonMaxTimeout,
} DivaSendVoiceEndReasons;

typedef enum
{
    DivaFaxFormatAutodetect = 0,
    DivaFaxFOrmatDefault = 0,
    DivaFaxFormatTIFF_ClassF = 1,
    DivaFaxFormatTIFF_ClassFSymmetric,
    DivaFaxFormatSFF,
    DivaFaxFormatTIFF_G3,
    DivaFaxFormatTIFF_G3Symmetric,
    DivaFaxFormatTIFF_G4,
    DivaFaxFormatTIFF_G4Symmetric,
    DivaFaxFormatColorJPEG = 100,
    DivaFaxFormatASCII = 200,
    DivaFaxFormatBinaryFile = 300,
} DivaFaxFormat;

typedef enum
{
    DivaFaxMaxSpeedAutomatic    = 0,
    DivaFaxMaxSpeed2400         = 2400,
    DivaFaxMaxSpeed4800         = 4800,
    DivaFaxMaxSpeed7200         = 7200,
    DivaFaxMaxSpeed9600         = 9600,
    DivaFaxMaxSpeed14400        = 14400,
    DivaFaxMaxSpeed33460        = 33600
} DivaFaxMaxSpeed;

typedef enum
{
    DivaFaxOptionsDefault               = 0x00000000,
    DivaFaxOptionDisableHighResolution  = 0x00000001,
    DivaFaxOptionDisableMR              = 0x00000002,
    DivaFaxOptionDisableMMR             = 0x00000004,
    DivaFaxOptionDisableECM             = 0x00000008,
    DivaFaxOptionEnablePolling          = 0x00000100,
    DivaFaxOptionRequestPolling         = 0x00000200,
    DivaFaxOptionReverseSession         = 0x00000400,
    DivaFaxOptionMultipleDocument       = 0x00000800,
    DivaFaxOptionEnableColor            = 0x00001000,
    DivaFaxOptionEnableInterrupt        = 0x00010000,
    DivaFaxOptionRequestInterrupt       = 0x00020000,
    DivaFaxOptionEnableClearChannel     = 0x00040000,
} DivaFaxOptions;

typedef enum
{
    DivaFaxResolutionStandard = 1,
    DivaFaxResolutionFine,
    DivaFaxResolutionSuperFine,
    DivaFaxResolutionUltraFine,
} DivaFaxResolution;

typedef enum
{
    DivaFaxHeadlineCodingASCII,
    DivaFaxHeadlineCodingUtf8,
    DivaFaxHeadlineCodingUtf16LE,
} DivaFaxHeadlineCoding;

typedef struct
{
    DWORD               Size;
    DWORD               Pages;
    DivaFaxResolution   FaxResolution;
    DWORD               ImageWidth;
    DWORD               ImageLength;
    DWORD               HorizontalResolution;
    DWORD               VerticalResolution;
    DWORD               Compression;
    DivaFaxFormat       Format;
    BOOL                AllPagesHaveSameFormat;
} DivaFaxDocumentProperties;

/*
 * Audio codec on the TDM / analog line.
 */
typedef enum
{
    LineAudio_ALaw          = 0,    // G.711 ALaw
    LineAudio_uLaw          = 1,    // G.711 uLaw

    /*
     * Codecs only available in direct access mode line configuration.
     */
    LineAudio_G722          = 10,   // G.722 codec
    LineAudio_G722_G711     = 11,   // G.722 codec, G.711 DTMF 
    LineAudio_PCM16_8KHz    = 20,   // PCM codec 16 Bit 8 KHz
    LineAudio_PCM16_16KHz   = 21,   // PCM codec 16 Bit 16 KHz
    LineAudio_PCM16_32KHz   = 23,   // PCM codec 16 Bit 32 KHz
    LineAudio_PCM16_48KHz   = 24,   // PCM codec 16 Bit 48 KHz
} DivaLineCodec;

/*
 * Codec used for exhange of data between application and Diva SDK
 */
typedef enum
{
    DivaDataCodecNone = 0,
    DivaDataCodecG729,
    DivaDataCodecG726,
    DivaDataCodecGSM,
    DivaDataCodecILBC,
    DivaDataCodecPCM16,
	DivaDataCodecG722,
} DivaDataCodec;

typedef enum
{
    DivaDCO_None                    = 0x00,
    DivaDCO_EnableVAD               = 0x01,
    DivaDCO_EnableComfortNoise      = 0x02,
    DivaDCO_EnableDTMFDetection     = 0x04,
    DivaDCO_EnableDTMFGeneration    = 0x08,
    DivaDCO_All                     = 0x0F,
} DivaDataCodecOptions;

typedef enum
{
    DivaSampleRate8000      = 0,
    DivaSampleRate16000     = 2,
    DivaSampleRate32000     = 4,
    DivaSampleRate48000     = 5,
} DivaSampleRates;

typedef enum
{   
    /*
     * Microsoft Windows waveform (WAV) formats:
     */
    DivaAudioFormat_aLaw8K8BitMono          =   0,
    DivaAudioFormat_uLaw8K8BitMono          =   1,
    DivaAudioFormat_PCM_8K8BitMono          =   2,
    DivaAudioFormat_PCM_8K16BitMono         =   3,

    /*
     * GSM 610 only for DivaMonitorRecordAudio
     */
    DivaAudioFormat_GSM_610                 =  10,
    DivaAudioFormat_G723_6_4                =  12,
    DivaAudioFormat_G723_5_3                =  13,

    /*
     * Raw audio formats:
     */
    DivaAudioFormat_Raw_aLaw8K8BitMono      = 100,
    DivaAudioFormat_Raw_uLaw8K8BitMono      = 101,
    DivaAudioFormat_Raw_PCM_8K8BitMono      = 102,
    DivaAudioFormat_Raw_PCM_8K16BitMono     = 103,
    DivaAudioFormat_Raw_ADPCM_8K4BitMono    = 104,
    DivaAudioFormat_Raw_ADPCM_6K4BitMono    = 105,

    /*
     * Compressed raw formats available for DivaMonitorRecordAudio
     * and if data codec is set accordingly.
     */
    DivaAudioFormat_Raw_GSM_610             = 200,
    DivaAudioFormat_Raw_G726                = 201,
    DivaAudioFormat_Raw_G729                = 202,
    DivaAudioFormat_Raw_ILBC                = 203,
    DivaAudioFormat_Raw_AMR_4_75            = 204,
    DivaAudioFormat_Raw_AMR_5_15            = 205,
    DivaAudioFormat_Raw_AMR_5_9             = 206,
    DivaAudioFormat_Raw_AMR_6_7             = 207,
    DivaAudioFormat_Raw_AMR_7_4             = 208,
    DivaAudioFormat_Raw_AMR_7_95            = 209,
    DivaAudioFormat_Raw_AMR_10_2            = 210,
    DivaAudioFormat_Raw_AMR_12_2            = 211,
    DivaAudioFormat_Raw_G723_6_4            = 212,
    DivaAudioFormat_Raw_G723_5_3            = 213,

    /*
     * HD Audio formats
     */
	DivaAudioFormat_Raw_G722                = 214,
    DivaAudioFormat_Raw_PCM_16K16Bit        = 215,
    DivaAudioFormat_Raw_PCM_32K16Bit        = 217,
    DivaAudioFormat_Raw_PCM_48K16Bit        = 218,


} DivaAudioFormat;

typedef enum
{
    DivaVoiceOptionDefault              = 0x0000,
    DivaVoiceOptionEarlyDataChannel     = 0x0001,
    DivaVoiceOptionEchoCanceller        = 0x0002,
    DivaVoiceOptionEarlyDataDiscOnInfo  = 0x0004,
} DivaVoiceOptions;

typedef enum
{
    DivaVoiceDataSourceFile         = 1,
    DivaVoiceDataSourceMemory
} DivaVoiceDataSource;

typedef enum
{
    DivaVoicePositionFormatBytes    = 1,
    DivaVoicePositionFormatMSec
} DivaVoicePositionFormat;

typedef struct
{
    DWORD                   Size;
    DivaVoicePositionFormat Format;
    int                     Offset;
} DivaVoicePosition;

typedef struct
{
    DWORD                   Size;
    DivaVoiceDataSource     DataSource;
    DivaAudioFormat         DataFormat;
    DivaVoicePositionFormat PositionFormat;
    DWORD                   StartOffset;
    DWORD                   Duration;
    union
    {
        struct
        {
            char*           pFilename;
        } File;
        struct
        {
            DWORD           DataLength;
            char*           pBuffer;
        } Memory;
    } Source;
} DivaVoiceDescriptor;

typedef enum
{
    /*
     * Refer to the Diva API documentation for information 
     * about tones that can be generated and detected.
     */
    DivaEndOfTone                   = 0x80,
    DivaUnknownTone                 = 0x81,
    DivaDialTone                    = 0x82,
    DivaPBXInternalDialTone         = 0x83,
    DivaSpecialDialTone             = 0x84,
    DivaSecondDialTone              = 0x85,
    DivaRingingTone                 = 0x86,
    DivaSpecialRingingTone          = 0x87,
    DivaBusyTone                    = 0x88,
    DivaCongestionTone              = 0x89,
    DivaSpecialInformationTone      = 0x8A,
    DivaComfortNoise                = 0x8B,
    DivaHoldTone                    = 0x8C,
    DivaRecordTone                  = 0x8D,
    DivaCallerWaitingTone           = 0x8E,
    DivaCallWaitingTone             = 0x8F,
    DivaPayTone                     = 0x90,
    DivaPositiveIndicationTone      = 0x91,
    DivaNegaitveIndicationTone      = 0x92,
    DivaWarningTone                 = 0x93,
    DivaIntrusionTone               = 0x94,
    DivaCallingCardServiceTone      = 0x95,
    DivaPayphoneRecognitionTone     = 0x96,
    DivaCPEAlertingSignal           = 0x97,
    DivaOffHookWarningTone          = 0x98,
    DivaSignalingSystem5ToneF1      = 0xAB,
    DivaSignalingSystem5ToneF2      = 0xAC,
    DivaSignalingSystem5ToneF1F2    = 0xAD,
    DivaInterceptTone               = 0xAF,
    DivaModemCallingTone            = 0xC0,
    DivaFaxCallingTone              = 0xC1,
    DivaAnswerTone                  = 0xC2,
    DivaAnswerTonePhaseReversal     = 0xC3,
    DivaANSam                       = 0xC4,
    DivaANSamPhaseReversal          = 0xC5,
    DivaBell103AnswerTone           = 0xC6,
    DivaFaxFlags                    = 0xC7,
    DivaFaxG2GroupId                = 0xC8,
    DivaHumanSpeech                 = 0xC9,
    DivaAnsweringMachineTone        = 0xCA,
    DivaToneAlertingSignal          = 0xCB,

    // Special Information tones
    DivaSITTone0                    = 0xA0,
    DivaSITTone1                    = 0xA1,
    DivaSITTone2                    = 0xA2,
    DivaSITTone3                    = 0xA3,
    DivaSITOperatorIntercept        = 0xA4,
    DivaSITVacantCircuit            = 0xA5,
    DivaSITRecoder                  = 0xA6,
    DivaSITNoCircuitFound           = 0xA7,


    // Tones for HOOK signaling 
    DivaHookDigit1                  = 0x10,
    DivaHookDigit2                  = 0x11,
    DivaHookDigit3                  = 0x12,
    DivaHookDigit4                  = 0x13,
    DivaHookDigit5                  = 0x14,
    DivaHookDigit6                  = 0x15,
    DivaHookDigit7                  = 0x16,
    DivaHookDigit8                  = 0x17,
    DivaHookDigit9                  = 0x18,
    DivaHookDigit0                  = 0x19,
    DivaHookToneFlash               = 0x1C,
    DivaHookToneRing                = 0x1D,
    DivaHookToneOffHook             = 0x1E,
    DivaHookToneOnHook              = 0x1F,

    DivaCustomToneBase              = 0x8000,
} DivaContinuousTones;

typedef enum
{
    DivaToneMFFallingEdge           = 0xF0,
    DivaToneMF1                     = 0xF1,
    DivaToneMF2                     = 0xF2,
    DivaToneMF3                     = 0xF3,
    DivaToneMF4                     = 0xF4,
    DivaToneMF5                     = 0xF5,
    DivaToneMF6                     = 0xF6,
    DivaToneMF7                     = 0xF7,
    DivaToneMF8                     = 0xF8,
    DivaToneMF9                     = 0xF9,
    DivaToneMF0                     = 0xFA,
    DivaToneMFStart                 = 0xFD,
    DivaToneMFStop                  = 0xFF,
} DivaMultiFrequencyTones;

typedef enum
{
    DivaR2ForwardToneOff            = 0xD0,
    DivaR2ForwardTone1              = 0xD1,
    DivaR2ForwardTone2              = 0xD2,
    DivaR2ForwardTone3              = 0xD3,
    DivaR2ForwardTone4              = 0xD4,
    DivaR2ForwardTone5              = 0xD5,
    DivaR2ForwardTone6              = 0xD6,
    DivaR2ForwardTone7              = 0xD7,
    DivaR2ForwardTone8              = 0xD8,
    DivaR2ForwardTone9              = 0xD9,
    DivaR2ForwardTone10             = 0xDA,
    DivaR2ForwardTone11             = 0xDB,
    DivaR2ForwardTone12             = 0xDC,
    DivaR2ForwardTone13             = 0xDD,
    DivaR2ForwardTone14             = 0xDE,
    DivaR2ForwardTone15             = 0xDF,

    DivaR2BackwardToneOff           = 0xB0,
    DivaR2BackwardTone1             = 0xB1,
    DivaR2BackwardTone2             = 0xB2,
    DivaR2BackwardTone3             = 0xB3,
    DivaR2BackwardTone4             = 0xB4,
    DivaR2BackwardTone5             = 0xB5,
    DivaR2BackwardTone6             = 0xB6,
    DivaR2BackwardTone7             = 0xB7,
    DivaR2BackwardTone8             = 0xB8,
    DivaR2BackwardTone9             = 0xB9,
    DivaR2BackwardTone10            = 0xBA,
    DivaR2BackwardTone11            = 0xBB,
    DivaR2BackwardTone12            = 0xBC,
    DivaR2BackwardTone13            = 0xBD,
    DivaR2BackwardTone14            = 0xBE,
    DivaR2BackwardTone15            = 0xBF,
} DivaR2Tones;

typedef struct
{
    DWORD       Frequency1;
    DWORD       Frequency1Variation;
    DWORD       Frequency2;
    DWORD       Frequency2Variation;
    DWORD       Duration;
    DWORD       DurationVariation;
    DWORD       Pause;
    DWORD       PauseVariation;
} DivaToneDefinition;

typedef enum
{
    DivaPayload_PCMU    = 0x00,
    DivaPayload_PCMA    = 0x08,
    DivaPayload_G723    = 0x04,
    DivaPayload_G726    = 0x02,     // G.726, 32 kBit/s
    DivaPayload_GSM     = 0x03,     // GSM 06.10 / ETS 300 961 full rate
} DivaPayloadProtocol;

typedef enum
{
    DivaDisableVoiceActivityDetection = 0x01,
    DivaDisableComfortNoise           = 0x02,
    DivaDisableDTMFDetection          = 0x04,
    DivaDisableDTMFGeneration         = 0x08,
    DivaG723LowCodingRate             = 0x10,
} DivaPayloadOptions;

typedef struct
{
    DWORD               Size;
    DivaPayloadProtocol PayloadProtocol;
    DWORD               PayloadOptions;
    DWORD               MaxPacketLateRate;
    DWORD               MaxDejitterDelay;
    DWORD               SSRC;
    DWORD               VoiceOptions;
} DivaVoIPParams;

typedef enum
{
    // protocols as defined in ES 201 912. 

    // protocol 1 uses data link frame as specified in EN 300 659-2 (channel 
    // seizure), and transfer layer as specified in GSM 03.40
    DivaSMSProtocolOne = 1,

    // protocol 2 uses data link frame as specified in EN 300 659-1 (NO channel
    // seizure), and transfer layer as specified in ES 201 912
    DivaSMSProtocolTwo,     // (reserved for future use)

} DivaSMSProtocol;

typedef struct
{
    DivaSMSProtocol Protocol;
} DivaSMSParams;

typedef enum
{
    DivaModemOptionsDefault                     = 0x00000000,
    DivaModemOption7BitsPerChar                 = 0x00000001,
    DivaModemOption2StopBits                    = 0x00000002,
    DivaModemOptionParityOdd                    = 0x00000004,
    DivaModemOptionParityEven                   = 0x00000008,
    DivaModemOptionDisableRetrain               = 0x00000010,
    DivaModemOptionDisableCompression           = 0x00000020,
    DivaModemOptionDisableV42                   = 0x00000040,
    DivaModemOptionDisableMNP                   = 0x00000080,
    DivaModemOptionForceReliable                = 0x00000100,

    DivaModemOptionDisableAnswerTone            = 0x00001000,
    DivaModemOptionEnableAutoMode               = 0x00002000,
    DivaModemOptionSDLCEnable                   = 0x00010000,

    /*
    * Modem options for synchronous modems. The options following
    * DivaModemOptionSynchronous are only valid if this bit is set.
    */
    DivaModemOptionSynchronous                  = 0x80000000,
    DivaModemOptionSyncV22Normal                = 0x80100000,
    DivaModemOptionSyncEnableV22bis             = 0x80200000,
    DivaModemOptionSyncEnableV22FastSetup       = 0x80400000,
    DivaModemOptionSyncEnableV22BisFastSetup    = 0x82000000,
    DivaModemOptionSyncEnableV29FastSetup       = 0x80800000,
    DivaModemOptionSyncSDLCExtModuloMode        = 0x81000000
} DivaModemOptions;

typedef enum
{
    DivaTransferOptionDefault           = 0x0000,
    DivaTransferOptionNoHoldRequired    = 0x0001,
    DivaTransferOptionOnAlerting        = 0x0002,
    DivaTransferOptionOnProceeding      = 0x0004,
    DivaTransferOptionCallDeflection    = 0x0008,
    DivaTransferOptionUseCallingNumber  = 0x0010,
    DivaTransferOptionUseCallingName    = 0x0020,
} DivaTransferOptions;

typedef enum
{
    DivaBChannelNotAssigned             = 0xffffffff
} DivaBChannel;

typedef struct
{
    DWORD           Size;               // The size of this structure
    DWORD           Channels;           // The amount of data channels
    BOOL            bModemSupported;    // If true analog modem is supported
    BOOL            bFaxSupported;      // If true fax is supported.
    BOOL            bVoIPSupport;       // If VoIP support is available
    DWORD           CapiControllerId;   // Ctrl Id assigned by CAPI
    DivaLineCodec   LineCodec;          // audio codec on the line
    /*
     * Available since version 3.0
     */
    BOOL            bLISupported;       // If line interconnect is supported
    DWORD           SerialNumber;       // Serial number of the card, zero if not avalaible.

    /*
     * Available since version 3.1
     */
    DWORD           HardwareLineIndex;  // If hardware supports multiple line definces, index starting with one.

    /*
     * Available since version 3.4
     */
    BOOL            bExtVoiceSupported; // V Series adapter.

    /*
     * Available after version 3.5.
     */
    BOOL            bHoldRetrieveSupported;
    BOOL            bTransferSupported;
    BOOL            bForwardSupported;
    BOOL            bCallDeflectionSupported;
    BOOL            bManagementSupported;
     
} DivaLineDeviceInfo;

typedef struct
{
    DWORD                   Size;
    // General parameters:
    char                    LocalNumber[MAX_ADDR_LEN];
    char                    LocalSubAddress[MAX_SUBADDR_LEN];
    // Fax-related parameters:
    char                    LocalFaxId[MAX_ADDR_LEN];
    char                    FaxHeadLine[MAX_ADDR_LEN];
    DivaFaxMaxSpeed         DefaultMaxSpeed;
    DWORD                   FaxOptions;
} DivaLineDeviceParamsFax;

typedef struct
{
    DWORD                   Size;
    // General parameters:
    char                    LocalNumber[MAX_ADDR_LEN];
    char                    LocalSubAddress[MAX_SUBADDR_LEN];
    // Voice-related parameters:
    DWORD                   VoiceOptions;
} DivaLineDeviceParamsVoice;

typedef struct
{
    DWORD                   Size;

    DWORD                   LineDevice;
    DivaCallState           CallState;
    DivaListenType          Service;
    DivaCallType            CallType;
    BOOL                    bDataTransferPossible;

    char                    CallingNumber[MAX_ADDR_LEN];
    char                    CalledNumber[MAX_ADDR_LEN];
    DWORD                   RxSpeed;
    DWORD                   TxSpeed;
    DWORD                   Compression;

    // parameter valid if call fails or after termination:
    DivaDisconnectReasons   DisconnectReason;
    DWORD                   dwISDNCause;

    // special parameters for CallTypeFax:
    char                    RemoteFaxId[MAX_ADDR_LEN];
    char                    LocalFaxId[MAX_ADDR_LEN];
    DWORD                   dwFaxPages;
    BOOL                    bPollingActive;
    DivaFaxResolution       PageResolution;
    BOOL                    bMRActive;
    BOOL                    bMMRActive;
    BOOL                    bECMActive;

    // special parameters for CallTypeVoice:
    BOOL                    bEchoCancellerActive;

    // supplementary services:
    DWORD                   dwRedirectReason;
    char                    RedirectedNumber[MAX_ADDR_LEN];
    char                    RedirectingNumber[MAX_ADDR_LEN];

    DivaSignalledCallType   SignalledCallType;

    /*
     * Available since version 3.0
     */
    DWORD                   AssignedBChannel;
} DivaCallInfo;

typedef enum
{
    DivaEventIncomingCall,
    DivaEventCallProgress,
    DivaEventCallInfo,
    DivaEventEarlyDataChannelConnected,
    DivaEventCallConnected,
    DivaEventCallDisconnected,
    DivaEventSendVoiceDone, // obsolete event
    DivaEventFaxPageSent,
    DivaEventFaxSent,
    DivaEventFaxPageReceived,
    DivaEventFaxReceived,
    DivaEventDataAvailable,
    DivaEventDataSent,
    DivaEventDTMFReceived,
    DivaEventSendDTMFToneEnded,
    DivaEventToneDetected,
    DivaEventSendToneEnded,
    DivaEventHoldCompleted,
    DivaEventRetrieveCompleted,
    DivaEventSetupTransferCompleted,
    DivaEventTransferCompleted,
    DivaEventReserved1,
    DivaEventRecordVoiceEnded,
    DivaEventSendVoiceEnded,
    DivaEventSendVoiceRestarted,
    DivaEventLIConnectCompleted,
    DivaEventLIDisconnected,
    DivaEventConferenceInfo,
    DivaEventConferenced,   // obsolete

    DivaEventSendVoiceCanceled,
    DivaEventCallDisconnectNotify,
    DivaEventCallHoldNotify,
    DivaEventCallRetrievedNotify,
    DivaEventCallTransferredNotify,
    DivaEventFaxDocumentSent,
    DivaEventDTMFMaxDigits,
    DivaEventDTMFTerminationDigit,
    DivaEventDTMFInterDigitTimeout,
    DivaEventDTMFInitialDigitTimeout,
    DivaEventDTMFMaxTimeout,
    DivaEventTransferRequested,
    DivaEventDataChannelStatus,
    DivaEventDetailedFaxStatus,
    DivaEventDataFrameStatus,
    DivaEventIPMediaChannelStatus,

    /*
     * Monitoring events.
     */
    DivaEventMonitorCallInitiated = 100,
    DivaEventMonitorCallConnected,
    DivaEventMonitorCallDisconnected,
    DivaEventMonitorCallInfo,
    DivaEventMonitorFrameReceived, 
    DivaEventMonitorRecordEnded,
    DivaEventMonitorStatus, 
    DivaEventMonitorAudioData,
    DivaEventMonitorDTMFDetected,
    DivaEventMonitorToneDetected,

    /*
     * MWI / Supplementary Services Events
     */
    DivaEventMWICompleted = 150,
    DivaEventMWIIndicated,
    DivaEventFlashCompleted,

    /*
     * Generic tone detection and generation
     */
    DivaEventGenericToneEnded = 200,
    DivaEventGenericToneDetected,
    DivaEventGenericToneInfo,
    DivaEventFSKDataDetected,
    DivaEventCustomToneDetected,

    /*
     * Timer services
     */
    DivaEventCallTimer = 250,
    DivaEventApplicationTimer,

    /*
     * Answering Machine detection
     */
    DivaEventAnsweringMachineDetector = 300,

    /*
     * SMS specific events
     */
    DivaEventSms1MsgReceived = 400,
    DivaEventSmsError,

    /*
     * Status reports for device and registrations
     */
    DivaEventDeviceStatusChanged = 500,
    DivaEventRegistrationStatus,

    /*
     * Events related to IP based protocols.
     */
    DivaEventSIPMessageReceived = 1000,

    /*
     * Events for speech engine support
     */
    DivaEventSpeechRecognizerStatus = 1100,
    DivaEventSpeechRecognizerProgress,

} DivaEvent;

// for backward compatibility:
#define DivaEventVoiceFileDone DivaEventSendVoiceDone


typedef void (*DivaCallbackT) (
    DivaAppHandle   hApp,
    DivaEvent       Event,
    PVOID           pEventSpecific1,
    PVOID           pEventSpecific2
    );

typedef void (*DivaCallbackExT) (
    PVOID           pContext,
    DivaEvent       Event,
    PVOID           pEventSpecific1,
    PVOID           pEventSpecific2
    );

typedef void (*DivaCallbackSignalT) (
    PVOID           pContext
    );


typedef void (*DivaStdCallbackT) (
    DivaAppHandle   hApp,
    DivaEvent       Event,
    PVOID           pEventSpecific1,
    PVOID           pEventSpecific2
    );

typedef void (*DivaStdCallbackExT) (
    PVOID           pContext,
    DivaEvent       Event,
    PVOID           pEventSpecific1,
    PVOID           pEventSpecific2
    );

typedef void (*DivaStdCallbackSignalT) (
    PVOID           pContext
    );




/*
 * Defines and data structures for conference
 */
typedef enum
{
    DivaConferencePropertyMaxMembers    = 1,    // Set Get
    DivaConferencePropertyOptions       = 2,    // Set Get
    DivaConferencePropertyDefRights     = 3,    // Set Get
    DivaConferencePropertyMemberRights  = 4,    // Set Get
    DivaConferencePropertySupervisor    = 5,    // Set
    DivaConferencePropertyNumMembers    = 6,    // Get
    DivaConferencePropertyMembers       = 7,    // Get
    DivaConferencePropertyNumTalkers    = 8,    // Get
    DivaConferencePropertyTalkers       = 9,    // Get
} DivaConferencePropertyType;

typedef enum
{
    DivaConferenceOptionLocal       = 1,
} DivaConferenceOptions;

typedef enum
{
    DivaConferenceRightSpeak        = 1,
    DivaConferenceRightListen       = 2,
    DivaConferenceRightSupervisor   = 7,
} DivaConferenceRights;

typedef struct
{
    DivaCallHandle                  hdCall;
    AppCallHandle                   haCall;
    DivaConferenceRights            Rights;
    BOOL                            bTalking;
} DivaConferenceMemberInfo;

typedef struct
{
    DivaCallHandle                  hdCall;
    DivaConferenceRights            Rights;
} DivaConferenceMemberRights;

typedef struct
{
    DivaCallHandle                  hdSupervisor;
    BOOL                            bEnable;
    DivaCallHandle                  hdTalkTo;
} DivaConferenceSupervisor;

typedef struct
{
    union 
    {
        DWORD                       MaxMembers;
        DivaConferenceOptions       Options;
        DivaConferenceRights        DefRights;
        DivaConferenceMemberRights  MemberRights;
        DivaConferenceSupervisor    Supervisor;
        DWORD                       NumMembers;
        DivaConferenceMemberInfo    Members;
        DWORD                       NumTalkers;
        DivaConferenceMemberInfo    Talkers;
    } T;
} DivaConferenceProperty;

typedef enum
{
    DivaConferenceStateIdle = 0,
    DivaConferenceStateOnHold,
    DivaConferenceStateAdding,
    DivaConferenceStateRemoving,
    DivaConferenceStateConnected
}DivaConferenceState;

#define MAX_CONF_MEMBERS    30

typedef struct
{
    // Size of the structure, may depend on the API version
    DWORD               Size;
    
    DivaConferenceState State;
    DWORD               MaxMembers;
    DWORD               CurrentMembers;

    DivaCallHandle      hdConfMembers[MAX_CONF_MEMBERS];
    AppCallHandle       haConfMembers[MAX_CONF_MEMBERS];
} DivaConferenceInfo;

typedef enum
{
    DivaVolumeMin = -18,
    DivaVolumeNormal = 0,
    DivaVolumeMax = +18
} DivaVolume;

typedef enum
{
    DivaSamplingRateMin     = 1250,
    DivaSamplingRateNormal  = 8000,
    DivaSamplingRateMax     = 51200
} DivaSamplingRate;

typedef enum DivaDirection
{
    DivaDirectionInbound  = 1,
    DivaDirectionOutbound = 2,
    DivaDirectionBoth     = 3
} DivaDirection;

typedef struct
{
    WORD            wYear;
    WORD            wMonth;
    WORD            wDayOfWeek;
    WORD            wDay;
    WORD            wHour;
    WORD            wMinute;
    WORD            wSecond;
    WORD            wMilliseconds;
} DivaTime;

typedef struct
{
    DivaTime        SetupTime;
    DivaTime        CallProcTime;
    DivaTime        AlertTime;
    DivaTime        ConnectTime;
    DivaTime        ConnectAckTime;
    DivaTime        DisconnectTime;
    DivaTime        ReleaseTime;
    DivaTime        ReleaseCompTime;
} DivaCallTimeStatistics;

typedef enum
{
    DivaDTMFToneEnded = '~',
} DivaDTMFTone;

typedef enum
{
    DivaProcessingGroupEvent = 1,
    DivaProcessingGroupSending,
    DivaProcessingGroupRecording,
} DivaProcessingGroup;

#define DivaTerminationDigit_None   0x000000000
#define DivaTerminationDigit_0      0x000000001
#define DivaTerminationDigit_1      0x000000002
#define DivaTerminationDigit_2      0x000000004
#define DivaTerminationDigit_3      0x000000008
#define DivaTerminationDigit_4      0x000000010
#define DivaTerminationDigit_5      0x000000020
#define DivaTerminationDigit_6      0x000000040
#define DivaTerminationDigit_7      0x000000080
#define DivaTerminationDigit_8      0x000000100
#define DivaTerminationDigit_9      0x000000200
#define DivaTerminationDigit_A      0x000000400
#define DivaTerminationDigit_B      0x000000800
#define DivaTerminationDigit_C      0x000001000
#define DivaTerminationDigit_D      0x000002000
#define DivaTerminationDigit_S      0x000004000
#define DivaTerminationDigit_H      0x000008000
#define DivaTerminationDigit_FAXCNG 0x000010000
#define DivaTerminationDigit_FAXCED 0x000020000
#define DivaTerminationDigit_ALL    0x00002FFFF

typedef enum
{
    DivaResultUserTerminated,
    DivaResultHumanTalker,
    DivaResultAnsweringMachine,
    DivaResultAnsweringMachineTone,
    DivaResultSilence,
    DivaResultFaxOrModem,
} DivaResultAnsweringMachineDetector;

typedef enum
{
    DivaProtocolH323 = 1,
    DivaProtocolSIP,
} DivaSignalingProtocols;

typedef enum
{
    DivaTransportUDP,
    DivaTransportTCP,
} DivaTransportProtocols;


typedef enum
{
    DivaParamNumIPDevices = 100,
    DivaParamDisableTDMDevices,
    DivaParamDisableConfActiveTalker,
    DivaParamConfActiveTalkerNotifyInterval,
    DivaParamConfDTMFClamping,
} DivaInitParameterTypes;

typedef union
{
    DWORD   NumIPDevices;
    BOOL    DisableTDMDevices;
    BOOL    DisableConfActiveTalker;
    DWORD   ConfActiveTalkerNotifyInterval;
    BOOL    ConfDTMFClamping;
} DivaInitParameterValue;

typedef enum
{
    DivaDeviceParamNetworkInterface = 1,
    DivaDeviceParamMaxChannels,
    DivaDeviceParamSignalingProtocol,
    DivaDeviceParamSignalingPort,
    DivaDeviceParamSignalingTransport,
    DivaDeviceParamMaxRegistrations,
    DivaDeviceParamProxy,
    DivaDeviceParamProxyUser,
    DivaDeviceParamProxyPassword,
    DivaDeviceParamProxyRealm,
    DivaDeviceParamEnableTranspAddr,
    DivaDeviceParamDefaultFromAddress,
} DivaDeviceInitParameterTypes;

typedef union
{
    char                    NetworkInterface[MAX_ADDR_LEN];
    DWORD                   MaxChannels;
    DivaSignalingProtocols  SignalingProtocol;
    DWORD                   SignalingPort;
    DivaTransportProtocols  SignalingTransport;
    DWORD                   MaxRegistrations;
    char                    Proxy[MAX_ADDR_LEN];
    char                    ProxyUser[MAX_ADDR_LEN];
    char                    ProxyPassword[MAX_ADDR_LEN];
    char                    ProxyRealm[MAX_ADDR_LEN];
    BOOL                    EnableTranspAddr;
    char                    DefaultFromAddress[MAX_ADDR_LEN];
} DivaDeviceInitParameterValue;

typedef enum
{
    DivaH323EndpointTerminal = 1,
    DivaH323EndpointGateway,
} DivaH323EndpointType;

typedef struct
{
    DWORD                   Size;
    char                    RegistrarAddress[MAX_ADDR_LEN];
    char                    DisplayName[MAX_ADDR_LEN];
    char                    Domain[MAX_ADDR_LEN];
    char                    User[MAX_ADDR_LEN];
    char                    AuthorizationRealm[MAX_ADDR_LEN];
    char                    AuthorizationUser[MAX_ADDR_LEN];
    char                    Password[MAX_ADDR_LEN];
    DWORD                   RefreshInterval;
} DivaSIPRegistrarParams;

typedef struct
{
    DWORD                   Size;
    char                    GatekeeperAddress[MAX_ADDR_LEN];
    DivaH323EndpointType    Type;
    char                    Number[MAX_ADDR_LEN];
    char                    Aliases[MAX_ADDR_LEN];
    char                    Prefixes[MAX_ADDR_LEN];
    char                    Name[MAX_ADDR_LEN];
    DWORD                   RefreshInterval;
} DivaH323GatekeeperParams;

typedef enum
{
    DivaRegStatusIdle,
    DivaRegStatusInitiated,
    DivaRegStatusRegistered,
    DivaRegStatusFailed,
    DivaRegStatusReleasing,
    DivaRegStatusReleased,
} DivaRegistrationStatus;

typedef enum
{
    DivaRegResultSuccess = 0,
    DivaRegResultNetworkError,
    DivaRegResultTimeout,
    DivaRegResultUnavailable,
    DivaRegResultUnknownUser,
    DivaRegResultPermissionDenied,
    DivaRegResultInvalidParameter,
    DivaRegResultRejected,
} DivaRegistrationResults;

typedef enum
{
    DivaDataChannelDown = 0,
    DivaDataChannelUp,
} DivaDataChannelStatus;

typedef enum
{
    DivaDataFrameReceiveStarted = 0,
    DivaDataFrameReceiveAborted,
} DivaDataFrameStatus;

/*------------------------------------------------------------------------------
 *
 * Declarations for Diva API interface functions
 *
------------------------------------------------------------------------------*/

DIVA_SERVER_API DWORD DivaGetVersion (
    void
    );

DIVA_SERVER_API DWORD DivaGetVersionEx (
    DWORD   *pProductVersionMajor,
    DWORD   *pProductVersionMinor,
    DWORD   *pServiceUpdate,
    DWORD   *pBuildVersion
    );

DIVA_SERVER_API DWORD DivaSetInitParameter (
    DivaInitParameterTypes          Type,
    DivaInitParameterValue*         pValue );

DIVA_SERVER_API DWORD DivaSetDeviceInitParameter (
    DWORD                           LineDevice,
    DivaDeviceInitParameterTypes    Type,
    DivaDeviceInitParameterValue*   pValue
    );

DIVA_SERVER_API DWORD DivaGetFirstIPLineDevice ( 
    DWORD   *pLineDevice
    );

DIVA_SERVER_API DWORD DivaInitialize (
    void
    );


DIVA_SERVER_API void DivaTerminate (
    void
    );


DIVA_SERVER_API void DivaLogPrintf (
    const char*         strApplication,
    const char*         strModule,
    const char*         strType,
    const char*         strFormat,
    ...
    );


DIVA_SERVER_API void DivaTracePrintf (
    char                TraceCategory,
    DWORD               TraceId,
    const char*         strApplication,
    const char*         strModule,
    const char*         strType,
    const char*         strFormat,
    ...
    );


DIVA_SERVER_API int DivaSelect (
    void
    );


DIVA_SERVER_API DWORD DivaGetNumLineDevices (
    DWORD*              pNumLine
    );


DIVA_SERVER_API DWORD DivaGetLineDeviceInfo (
    DWORD               LineDeviceId,
    DivaLineDeviceInfo* pInfo
    );

DIVA_SERVER_API BOOL  DivaCheckDeviceCapabilities (
    DWORD               LineDeviceId,
    DWORD               Capability
    );

typedef enum
{
    DivaDevCapsPSTNBased = 100,
    DivaDevCapsIPBased,
    DivaDevCapsAnalogBased,
    DivaDevCapsExtEquipment = 200,
} DivaDeviceCapabilities;

DIVA_SERVER_API DWORD DivaRegister (
    DivaAppHandle*      pHandle,
    DWORD               EventMode,
    PVOID               EventModeSpecific1,
    PVOID               EventModeSpecific2,
    DWORD               MaxConn,
    DWORD               RxBuffers,
    DWORD               MaxBufferSize
    );

DIVA_SERVER_API DWORD DivaUnregister (
    DivaAppHandle       Handle
    );

DIVA_SERVER_API DWORD DivaRegisterSIPRegistrar (
    DivaAppHandle               hApp,
    DWORD                       LineDevice,
    DivaHandle                  hAppRegistrar,
    DivaHandle *                phDivaRegistrar,
    DivaSIPRegistrarParams *    pParams
    );

DIVA_SERVER_API DWORD DivaReleaseSIPRegistrar (
    DivaHandle                  hDivaRegistrar
    );

DIVA_SERVER_API DWORD DivaRegisterH323Gatekeeper (
    DivaAppHandle               hApp,
    DWORD                       LineDevice,
    DivaHandle                  hAppGatekeeper,
    DivaHandle *                phDivaGatekeeper,
    DivaH323GatekeeperParams *  pParams
    );

DIVA_SERVER_API DWORD DivaReleaseH323Gatekeeper(
    DivaHandle                  hDivaRegistrar
    );

DIVA_SERVER_API DWORD DivaGetRegistrationResult (
    DivaHandle                  hDivaRegistrar,
    DWORD *                     pResult,
    BOOL *                      pRetry );

DIVA_SERVER_API DWORD DivaCloseRegistration ( 
    DivaHandle                  hDivaRegistrar
    );

DIVA_SERVER_API DWORD DivaSetH323Gateway (
    DWORD                       LineDevice,
    const char *                pGatewayAddress
    );


typedef enum
{
    DivaExtensionFaxFormats = 1,
} DivaExtensions;

DIVA_SERVER_API DWORD DivaEnableExtensions ( 
    DivaAppHandle               hApp,
    DWORD                       LineDevice, 
    DivaExtensions              Type
    );

DIVA_SERVER_API DWORD DivaDisableExtensions ( 
    DivaAppHandle               hApp,
    DWORD                       LineDevice,
    DivaExtensions              Type
    );


DIVA_SERVER_API DWORD DivaSetLineDeviceParamsFax (
    DivaAppHandle               hApp,
    DWORD                       LineDeviceId,
    DivaLineDeviceParamsFax*    pParams
    );


DIVA_SERVER_API DWORD DivaSetLineDeviceParamsVoice (
    DivaAppHandle               hApp,
    DWORD                       LineDeviceId,
    DivaLineDeviceParamsVoice*  pParams
    );


DIVA_SERVER_API DWORD DivaListen (
    DivaAppHandle       hApp,
    DivaListenType      ServiceType,
    DWORD               LineDevice,
    char*               pCalledNumber
    );

DIVA_SERVER_API DWORD DivaListenChannel (
    DivaAppHandle   hApp,
    DivaListenType  ServiceType,
    DWORD           LineDevice,
    DWORD           Channel,
    char*           pCalledNumber
    );

DIVA_SERVER_API DWORD DivaCreateCall (
    DivaAppHandle       hApp,
    AppCallHandle       haCall,
    DivaCallHandle*     phdCall
    );

DIVA_SERVER_API DWORD DivaAttachToCall (
    DivaCallHandle      hdCall,
    AppCallHandle       haCall
    );


DIVA_SERVER_API DWORD DivaReject (
    DivaCallHandle      hdCall,
    BOOL                bOtherMayTakeIt
    );

DIVA_SERVER_API DWORD DivaProceeding (
    DivaCallHandle      hdCall,
    AppCallHandle       haCall
    );

DIVA_SERVER_API DWORD DivaAlert (
    DivaCallHandle      hdCall,
    AppCallHandle       haCall
    );

DIVA_SERVER_API DWORD DivaAnswer (
    DivaCallHandle      hdCall,
    AppCallHandle       haCall,
    DivaCallType        CallType
    );


DIVA_SERVER_API DWORD DivaAnswerVoice (
    DivaCallHandle      hdCall,
    AppCallHandle       haCall,
    DWORD               VoiceOptions
    );


DIVA_SERVER_API DWORD DivaAnswerVoIP (
    DivaCallHandle      hdCall,
    AppCallHandle       haCall,
    DivaVoIPParams*     VoIPParams
    );


DIVA_SERVER_API DWORD DivaAnswerFax (
    DivaCallHandle      hdCall,
    AppCallHandle       haCall,
    char*               pLocalFaxId,
    char*               pHeadLine,
    DivaFaxMaxSpeed     MaxSpeed,
    DWORD               FaxOptions
    );


DIVA_SERVER_API DWORD DivaAnswerModem (
    DivaCallHandle      hdCall,
    AppCallHandle       haCall,
    DWORD               MaxSpeed,
    DWORD               ModemOptions
    );

DIVA_SERVER_API DWORD DivaAnswerSMS (
    DivaCallHandle      hdCall,
    AppCallHandle       haCall,
    DivaSMSProtocol     Protocol
    );

DIVA_SERVER_API DWORD DivaConnect (
    DivaAppHandle       hApp,
    AppCallHandle       haCall,
    DivaCallHandle*     phdCall,
    char*               DestinationNumber,
    DivaCallType        CallType,
    DWORD               LineDevice
    );


DIVA_SERVER_API DWORD DivaConnectVoice (
    DivaAppHandle       hApp,
    AppCallHandle       haCall,
    DivaCallHandle*     phdCall,
    char*               DestinationNumber,
    DWORD               LineDevice,
    char*               LocalNumber,
    char*               LocalSubAddress,
    DWORD               VoiceOptions
    );


DIVA_SERVER_API DWORD DivaConnectVoIP (
    DivaAppHandle       hApp,
    AppCallHandle       haCall,
    DivaCallHandle*     phdCall,
    char*               DestinationNumber,
    DWORD               LineDevice,
    char*               LocalNumber,
    char*               LocalSubAddress,
    DivaVoIPParams*     VoIPParams
    );


DIVA_SERVER_API DWORD DivaConnectFax (
    DivaAppHandle       hApp,
    AppCallHandle       haCall,
    DivaCallHandle*     phdCall,
    char*               DestinationNumber,
    DWORD               LineDevice,
    char*               LocalNumber,
    char*               LocalSubAddress,
    char*               pLocalFaxId,
    char*               pHeadLine,
    DivaFaxMaxSpeed     MaxSpeed,
    DWORD               FaxOptions
    );


DIVA_SERVER_API DWORD DivaConnectModem (
    DivaAppHandle       hApp,
    AppCallHandle       haCall,
    DivaCallHandle*     phdCall,
    char*               DestinationNumber,
    DWORD               LineDevice,
    char*               LocalNumber,
    char*               LocalSubAddress,
    DWORD               MaxSpeed,
    DWORD               ModemOptions
    );

DIVA_SERVER_API DWORD DivaConnectSMS (
    DivaAppHandle       hApp,
    AppCallHandle       haCall,
    DivaCallHandle*     phdCall,
    char*               DestinationNumber,
    DWORD               LineDevice,
    char*               LocalNumber,
    DivaSMSProtocol     Protocol
    );

DIVA_SERVER_API DWORD DivaDial (
    DivaCallHandle      hdCall,
    char*               DestinationNumber
    );

DIVA_SERVER_API DWORD DivaSendInfo (
    DivaCallHandle      hdCall,
    unsigned char*      Info,           // reserved for future use
    DWORD               InfoLength      // reserved for future use
    );

DIVA_SERVER_API DWORD DivaSendFlash (
    DivaCallHandle      hdCall,
    DWORD               FlashLength
    );


DIVA_SERVER_API DWORD DivaSetCallType (
    DivaCallHandle      hdCall,
    DivaCallType        CallType
    );


DIVA_SERVER_API DWORD DivaSetCallTypeFax (
    DivaCallHandle      hdCall,
    char*               pLocalFaxId,
    char*               pHeadLine,
    DivaFaxMaxSpeed     MaxSpeed,
    DWORD               FaxOptions
    );


DIVA_SERVER_API DWORD DivaSetCallTypeVoice (
    DivaCallHandle      hdCall,
    DWORD               VoiceOptions
    );


DIVA_SERVER_API DWORD DivaSetCallTypeVoIP (
    DivaCallHandle      hdCall,
    DivaVoIPParams*     VoIPParams
    );


DIVA_SERVER_API DWORD DivaDisconnect (
    DivaCallHandle      hdCall
    );


DIVA_SERVER_API DWORD DivaAbortCall (
    DivaCallHandle      hdCall
    );


DIVA_SERVER_API DWORD DivaGetCallInfo (
    DivaCallHandle      hdCall,
    DivaCallInfo*       pCallInfo
    );


DIVA_SERVER_API DWORD DivaCloseCall (
    DivaCallHandle      hdCall
    );

DIVA_SERVER_API BOOL DivaGetEvent (
    DivaAppHandle       hApp,
    DivaEvent*          pEvent,
    PVOID*              ppEventSpecific1,
    PVOID*              ppEventSpecific2
    );

DIVA_SERVER_API DWORD DivaEnableDataChannel (
    DivaCallHandle      hdCall,
    BOOL                bEnable
    );

DIVA_SERVER_API DWORD DivaSendData (
    DivaCallHandle      hdCall,
    unsigned char*      pData,
    DWORD               DataLength,
    DWORD               DataHandle
    );


DIVA_SERVER_API DWORD DivaReceiveData (
    DivaCallHandle      hdCall,
    unsigned char*      pData,
    DWORD               BufferSize,
    DWORD*              pDataReceived
    );

typedef enum
{
    DivaDataOptionQualifier = 0x00000001,
    DivaDataOptionModeData  = 0x00000002,
    DivaDataOptionMoreData  = 0x00000002,
    DivaDataOptionDelivery  = 0x00000004,
} DivaDataOptions;

DIVA_SERVER_API DWORD DivaSendFrame (
    DivaCallHandle      hdCall,
    unsigned char*      pData,
    DWORD               DataLength,
    DWORD               DataHandle,
    DWORD               DataOptions
    );


DIVA_SERVER_API DWORD DivaReceiveFrame (
    DivaCallHandle      hdCall,
    unsigned char*      pData,
    DWORD               BufferSize,
    DWORD*              pDataReceived,
    DWORD*              pDataOptions
    );


DIVA_SERVER_API DWORD DivaSendFax (
    DivaCallHandle      hdCall,
    char*               pFilename,
    DivaFaxFormat       Format
    );


DIVA_SERVER_API DWORD DivaAppendFax (
    DivaCallHandle      hdCall,
    char                *pFilename,
    DivaFaxFormat       Format,
    BOOL                bNewDocument
    );


DIVA_SERVER_API DWORD DivaSendMultipleFaxFiles (
    DivaCallHandle      hdCall,
    int                 nFiles,
    char**              ppFileArray,
    DivaFaxFormat       Format
    );


DIVA_SERVER_API DWORD DivaAppendFaxFiles (
    DivaCallHandle      hdCall,
    int                 NumFiles,
    char**              ppFileArray,
    DivaFaxFormat       Format,
    BOOL                bNewDocument
    );


DIVA_SERVER_API DWORD DivaReceiveFax (
    DivaCallHandle      hdCall,
    char*               pFilename,
    DivaFaxFormat       Format
    );

DIVA_SERVER_API DWORD DivaReceiveFaxToMemory (
    DivaCallHandle      hdCall,
    DivaFaxFormat       Format
    );

DIVA_SERVER_API DWORD DivaReadFaxData (
    DivaCallHandle      hdCall, 
    unsigned char*      pBuffer,
    DWORD               BufferSize,
    DWORD*	            pBytesRead
    );

DIVA_SERVER_API DWORD DivaValidateFaxFile (
    const char*                 pFilename,
    BOOL*                       pRequiresExtensions,
    DivaFaxDocumentProperties*  pProperties
    );


DIVA_SERVER_API DWORD DivaSendVoiceFile (
    DivaCallHandle      hdCall,
    char*               pFilename,
    BOOL                bContinuous
    );


DIVA_SERVER_API DWORD DivaSendMultipleVoiceFiles (
    DivaCallHandle      hdCall,
    int                 nFiles,
    char**              ppFileArray,
    BOOL                bContinuous
    );


DIVA_SERVER_API DWORD DivaSendVoiceEx (
    DivaCallHandle          hdCall,
    DWORD                   NumObjects,
    DivaVoiceDescriptor*    pDescriptor,
    BOOL                    bContinuous,
    DWORD                   MaxSeconds
    );

DIVA_SERVER_API DWORD DivaAppendVoice (
    DivaCallHandle          hdCall,
    DivaVoiceDescriptor*    pDescriptor
    );


DIVA_SERVER_API DWORD DivaStopSendingVoiceFile (
    DivaCallHandle      hdCall
    );

DIVA_SERVER_API DWORD DivaStopSending (
    DivaCallHandle      hdCall
    );

DIVA_SERVER_API DWORD DivaPauseSend (
    DivaCallHandle      hdCall
    );

DIVA_SERVER_API DWORD DivaContinueSend (
    DivaCallHandle      hdCall
    );

DIVA_SERVER_API DWORD DivaForwardSend (
    DivaCallHandle      hdCall,
    DivaVoicePosition*  pPosition
    );

DIVA_SERVER_API DWORD DivaRewindSend (
    DivaCallHandle      hdCall,
    DivaVoicePosition*  pPosition
    );

DIVA_SERVER_API DWORD DivaGetSendPosition (
    DivaCallHandle      hdCall,
    DivaVoicePosition*  pPosition
    );

DIVA_SERVER_API DWORD DivaEnableRxData (
    DivaCallHandle      hdCall,
    BOOL                bEnable
    );

DIVA_SERVER_API DWORD DivaEnableTransactionRecording (
    DivaCallHandle      hdCall,
    BOOL                bEnable
    );

DIVA_SERVER_API DWORD DivaRecordVoiceFile (
    DivaCallHandle      hdCall,
    char*               pFilename,
    DivaAudioFormat     Format,
    DWORD               MaxRecordTime
    );

DIVA_SERVER_API DWORD DivaRecordAppendVoiceFile (
    DivaCallHandle      hdCall,
    char*               pFilename,
    DivaAudioFormat     Format,
    DWORD               MaxRecordTime
    );

DIVA_SERVER_API DWORD DivaGetVoiceFileLength (
    char*                   pFilename,
    DivaAudioFormat         Format,
    DivaVoicePositionFormat PositionFormat,
    DWORD*                  Length
    );

DIVA_SERVER_API DWORD DivaSetVoiceFileLength (
    char*                   pFilename,
    DivaAudioFormat         Format,
    DivaVoicePositionFormat PositionFormat,
    DWORD                   Length
    );

DIVA_SERVER_API DWORD DivaStopRecording (
    DivaCallHandle      hdCall
    );

DIVA_SERVER_API DWORD DivaPauseRecording (
    DivaCallHandle      hdCall
    );

DIVA_SERVER_API DWORD DivaContinueRecording (
    DivaCallHandle      hdCall
    );

DIVA_SERVER_API DWORD DivaGetRecordPosition (
    DivaCallHandle      hdCall,
    DivaVoicePosition   *pPosition
    );

DIVA_SERVER_API DWORD DivaReceiveAudio (
    DivaCallHandle      hdCall, 
    unsigned char       *pData, 
    DWORD               BufferSize,
    DWORD               *pDataReceived, 
    DivaAudioFormat     Format );

DIVA_SERVER_API DWORD   DivaSetVolume (
    DivaCallHandle      hdCall,
    DivaVolume          Volume,
    DivaDirection       Direction
    );

DIVA_SERVER_API DWORD   DivaSetSamplingRate (
    DivaCallHandle      hdCall,
    DWORD               SamplingRate,
    DivaDirection       Direction
    );

DIVA_SERVER_API DWORD DivaEnableEchoCanceller (
    DivaCallHandle      hdCall,
    BOOL                bEnable
    );
 
DIVA_SERVER_API DWORD DivaEnableAnsweringMachineDetector (
    DivaCallHandle      hdCall,
    DWORD               MaxInitialSilence, 
    DWORD               MaxHumanSpeakerTime ,
    DWORD               MaxInterSpeakerTimeout
    );

DIVA_SERVER_API DWORD DivaDisableAnsweringMachineDetector (
    DivaCallHandle      hdCall
    );

DIVA_SERVER_API DWORD DivaReportDTMF (
    DivaCallHandle      hdCall,
    BOOL                bEnable
    );

DIVA_SERVER_API DWORD DivaSendDTMF (
    DivaCallHandle      hdCall,
    char*               pTones
    );

DIVA_SERVER_API DWORD DivaSetDTMFProcessingRules (
    DivaCallHandle      hdCall,
    DivaProcessingGroup Group,
    DWORD               TerminationDigitMask,
    DWORD               MaxDigits,
    DWORD               InterDigitTimeout,
    DWORD               InitialDigitTimeout,
    DWORD               MaxTimeout
    );

DIVA_SERVER_API DWORD DivaGetDTMFBuffer (
    DivaCallHandle      hdCall,
    char*               Buffer,
    DWORD               BufferSize
    );

DIVA_SERVER_API DWORD DivaClearDTMFBuffer (
    DivaCallHandle      hdCall
    );

DIVA_SERVER_API DWORD DivaReportTones (
    DivaCallHandle      hdCall,
    BOOL                bEnable
    );


DIVA_SERVER_API DWORD DivaSendTone (
    DivaCallHandle              hdCall,
    DWORD                       NumTones,
    DivaMultiFrequencyTones*    pTones
    );


DIVA_SERVER_API DWORD DivaSendContinuousTone (
    DivaCallHandle      hdCall,
    DivaContinuousTones Tone,
    DWORD               MaxSeconds
    );


DIVA_SERVER_API DWORD DivaStopContinuousTone (
    DivaCallHandle      hdCall
    );


DIVA_SERVER_API DWORD DivaHold (
    DivaCallHandle      hdCall
    );


DIVA_SERVER_API DWORD DivaRetrieve (
    DivaCallHandle      hdCall
    );


DIVA_SERVER_API DWORD DivaSetupCallTransfer (
    DivaCallHandle      hdCall,
    AppCallHandle       haConsultCall,
    DivaCallHandle*     phdConsultCall,
    char*               DestinationNumber
    );


DIVA_SERVER_API DWORD DivaCompleteCallTransfer (
    DivaCallHandle      hdCall,
    DivaCallHandle      hdConsultCall,
    DWORD               TransferOptions
    );


DIVA_SERVER_API DWORD DivaBlindCallTransfer (
    DivaCallHandle      hdCall,
    char*               DestinationNumber,
    DWORD               TransferOptions
    );

DIVA_SERVER_API DWORD DivaAcceptCallTransfer (
    DivaCallHandle      hdCall
    );

DIVA_SERVER_API DWORD DivaRejectCallTransfer (
    DivaCallHandle      hdCall,
    DWORD               Reason
    );

DIVA_SERVER_API DWORD DivaLIConnect (
    DivaCallHandle      hdMainCall,
    DivaCallHandle      hdCall
    );


DIVA_SERVER_API DWORD DivaLIDisconnect (
    DivaCallHandle      hdMainCall,
    BOOL                bDisconnectCalls
    );


DIVA_SERVER_API DWORD DivaLIEnableRxData (
    DivaCallHandle      hdCall,
    BOOL                bEnable
    );

DIVA_SERVER_API DWORD DivaCreateConference (
    DivaAppHandle       hApp,
    DivaCallHandle      hdCall,
    AppCallHandle       haConference,
    DivaCallHandle      *phdConference
    );


DIVA_SERVER_API DWORD DivaDestroyConference (
    DivaCallHandle      hdConference,
    BOOL                bDisconnectCalls
    );

DIVA_SERVER_API DWORD DivaConferenceSetProperties (
    DivaCallHandle          hdConference,
    DWORD                   PropertyType,
    DivaConferenceProperty  *PropertyValue
    );

DIVA_SERVER_API DWORD DivaConferenceGetProperties (
    DivaCallHandle          hdConference,
    DWORD                   PropertyType,
    DivaConferenceProperty  *PropertyValue,
    DWORD                   PropertyValueSize,
    DWORD                   *pPropertyValueSizeUsed
    );

DIVA_SERVER_API DWORD DivaConferenceEnableRxData (
    DivaCallHandle          hdConference,
    BOOL                    bEnable
    );

DIVA_SERVER_API DWORD DivaAddToConference (
    DivaCallHandle      hdConference,
    DivaCallHandle      hdCall
    );


DIVA_SERVER_API DWORD DivaRemoveFromConference (
    DivaCallHandle      hdConference,
    DivaCallHandle      hdCall
    );

DIVA_SERVER_API DWORD DivaGetConferenceInfo (
    DivaCallHandle      hdConference,
    DivaConferenceInfo  *pConferenceInfo
    );

DIVA_SERVER_API BOOL DivaIsConferenceMaster ( 
    DivaCallHandle      hdConference,
    DivaCallHandle      hdCall
    );

DIVA_SERVER_API DWORD DivaSetConferenceMaster (
    DivaCallHandle      hdConference,
    DivaCallHandle      hdCall
    );

/*------------------------------------------------------------------------------
 *
 * Declarations for Diva API flexible call parameter. Requires 
 * understanding of the underlying technology, ISDN and / or Modem.
 *
------------------------------------------------------------------------------*/
/*
 * DivaCallPropertyType
 */
typedef enum
{
    /*
     * Common call properties for all types of calls
    */
    DivaCPT_CallType = 1,               // Get Set
    DivaCPT_LineDevice,                 // Get Set
    DivaCPT_SignaledService,            // Get Set
    DivaCPT_BearerCapabilities,         // Get Set
    DivaCPT_CalledNumber,               // Get
    DivaCPT_CallingNumber,              // Get Set
    DivaCPT_CalledNumberParams,         // Get Set
    DivaCPT_CallingNumberParams,        // Get Set
    DivaCPT_RedirectingNumber,          // Get
    DivaCPT_RedirectedNumber,           // Get
    DivaCPT_TxSpeed,                    // Get
    DivaCPT_RxSpeed,                    // Get

    DivaCPT_DiscReason,                 // Get
    DivaCPT_SignaledLineDiscReason,     // Get
    DivaCPT_RejectReason,               // Set

    /*
     * Voice parameter
     */
    DivaCPT_VoiceEchoCanceler,          // Set
    DivaCPT_VoiceEarlyDataChannel,      // Set
    DivaCPT_VoiceRecordSilenceTimeout,  // Set

    /*
     * 2nd calling party number for SMS
     */
    DivaCPT_SecondCallingNumber,        // Get
    DivaCPT_SecondCallingNumberParams,  // Get

    /*
     * Calling / connected name
     */
    DivaCPT_CallingName,                // Get Set
    DivaCPT_ConnectedName,              // Get

    /*
     * Calling / called sub address
     */
    DivaCPT_CallingSubAddress,          // Get Set
    DivaCPT_CalledSubAddress,           // Get Set
    DivaCPT_OriginalCalledNumber,       // Get
    DivaCPT_ConnectedNumber,            // Get Set
    DivaCPT_CalledName,                 // Get
    DivaCPT_DataChannel,                // Get Set

    /*
     * Disconnect reason and disconnect cause
     */
    DivaCPT_DisconnectReason,           // Set
    DivaCPT_DisconnectCause,            // Set

    /*
     * Redirection number, to which number has the 
     * call been deverted.
     */
    DivaCPT_RedirectionNumber,          // Get
    DivaCPT_RedirectReason,             // Get Set

    /*
     * Streaming properties.
     */
    DivaCPT_VoiceRecordStartTones = 100,// Set
    DivaCPT_VoiceDTMF_SendDuration,     // Set
    DivaCPT_VoiceDTMF_SendPause,        // Set
    DivaCPT_VoiceDTMF_DetectDuration,   // Set
    DivaCPT_VoiceDTMF_DetectPause,      // Set
    DivaCPT_VoiceRemoveDTMFFromStream,  // Set
    DivaCPT_VoiceEarlyDataDiscOnInfo,   // Set
    DivaCPT_EchoCancellerEnableNLP,     // Set
    DivaCPT_EchoCancellerAutoDisable1,  // Set
    DivaCPT_EchoCancellerAutoDisable2,  // Set
    DivaCPT_EchoCancellerTailLength,    // Set
    DivaCPT_EchoCancellerPreDelay,      // Set
    DivaCPT_EnableDTMFTrailingEdge,     // Set
    DivaCPT_DataCodec,                  // Set
    DivaCPT_DataCodecOptions,           // Set
    DivaCPT_DataCodecSamplesPerPacket,  // Set
    DivaCPT_DisableMFDetection,         // Set
    DivaCPT_EnableHookSignaling,        // Set
    DivaCPT_EnableR2ForwardDetection,   // Set
    DivaCPT_EnableR2BackwardDetection,  // Set
    DivaCPT_FSKData,                    // Get
    DivaCPT_DTMFMode,                   // Set
    DivaCPT_EnableTransparentLI,        // Set
    DivaCPT_HumanTalkerThreshold,       // Set
    DivaCPT_VoiceActivityThreshold,     // Set
    DivaCPT_VoiceDTMF_TxLevelGroup,     // Set
    DivaCPT_SingleToneOffDuration,      // Set
    DivaCPT_DualToneOffDuration,        // Set
    DivaCPT_HookSignalingOffhookTime,   // Set
    DivaCPT_HookSignalingHookFlashTime, // Set
    DivaCPT_ToneDetectorFFTLength,      // Set
    DivaCPT_SingleToneDetectorMinFreq,  // Set
    DivaCPT_SingleToneDetectorMaxFreq,  // Set
    DivaCPT_DataCodecSampleRate,        // Set

    /*
     * Fax parameter
     */
    DivaCPT_FaxLocalId = 200,           // Set
    DivaCPT_FaxHeadline,                // Set
    DivaCPT_FaxRemoteId,                // Get
    DivaCPT_FaxPages,                   // Get
    DivaCPT_FaxMaxSpeed,                // Set
    DivaCPT_FaxHighResolution,          // Set
    DivaCPT_FaxEnablePolling,           // Set
    DivaCPT_FaxReverseSession,          // Set
    DivaCPT_FaxMultiDocument,           // Set
    DivaCPT_FaxDisableECM,              // Set
    DivaCPT_FaxDisableMR,               // Set
    DivaCPT_FaxDisableMMR,              // Set
    DivaCPT_FaxPageQuality,             // Get
    DivaCPT_FaxPageEndInfo,             // Get
    DivaCPT_FaxRemoteFeatures,          // Get
    DivaCPT_FaxRemoteMaxHorzRes,        // Get
    DivaCPT_FaxRemoteMaxVertRes,        // Get
    DivaCPT_FaxRemoteMaxSpeed,          // Get
    DivaCPT_FaxRemoteNSF,               // Get
    DivaCPT_FaxLocalNSF,                // Set
    DivaCPT_FaxEnableColor,             // Set
    DivaCPT_FaxColorSelected,           // Get
    DivaCPT_EnableInterrupt,            // Set
    DivaCPT_RequestInterrupt,           // Set
    DivaCPT_FaxProcedureInterrupt,      // Get
    DivaCPT_FaxEnableSecurity,          // Set
    DivaCPT_FaxRemoteSupportsSubaddr,   // Get
    DivaCPT_FaxRemoteSupportsPassword,  // Get
    DivaCPT_FaxSignalSubAddress,        // Set
    DivaCPT_FaxSignalPassword,          // Set
    DivaCPT_FaxRemoteSubAddress,        // Get
    DivaCPT_FaxRemotePassword,          // Get
    DivaCPT_FaxDisableFileBuffering,    // Set
    DivaCPT_FaxUseTextForSending,       // Set
    DivaCPT_FaxAllowDocumentStretching, // Set
    DivaCPT_FaxRemoteScanLineLength,    // Get
    DivaCPT_FaxStoreMode,               // Set
    DivaCPT_FaxPassDataOnNextPageStart, // Set
    DivaCPT_FaxStartPage,               // Set
    DivaCPT_FaxEnableClearChannel,      // Set
    DivaCPT_EnableFaxStatusReporting,   // Set
    DivaCPT_FaxReportDCS,               // Get
    DivaCPT_FaxReportTrainingResult,    // Get
    DivaCPT_FaxReportTrainingStats,     // Get
    DivaCPT_FaxReportPageQuality,       // Get
    DivaCPT_FaxReportPartialPage,       // Get
    DivaCPT_FaxReportT30Timeout,        // Get
    DivaCPT_FaxT30Phase,                // Get
    DivaCPT_FaxResultReport,            // Get
    DivaCPT_FaxEnableBinaryFileTransfer,// Set
    DivaCPT_FaxRequestBinaryFileTransfer,//Set
    DivaCPT_FaxBinaryFileTransferActive,// Get
    DivaCPT_FaxSelectManualResult,      // Set
    DivaCPT_FaxManualResultValue,       // Set
    DivaCPT_FaxUseHeadlineFont,         // Set
    DivaCPT_FaxHeadlinePageInfo,        // Set
    DivaCPT_FaxHeadlineFontFile,        // Set
    DivaCPT_FaxHeadlineFontFace,        // Set
    DivaCPT_FaxHeadlineFontSizePoints,  // Set
    DivaCPT_FaxHeadlineCoding,          // Set
    DivaCPT_FaxHeadlineOmitDateTime,    // Set
    DivaCPT_FaxHeadlineOmitPageInfo,    // Set

    /*
     * VoIP / RTP Parameter
     */
    DivaCPT_RTPPayloadProtocol = 300,   // Set
    DivaCPT_RTPPayloadOptions,          // Set
    DivaCPT_RTPMaxPacketLateRate,       // Set
    DivaCPT_RTPMaxDejitterDelay,        // Set
    DivaCPT_RTPSSRC,                    // Set
    DivaCPT_RTPLocalIpAddress,          // Set
    DivaCPT_RTPLocalPort,               // Set
    DivaCPT_RTPRemoteIpAddress,         // Set
    DivaCPT_RTPRemotePort,              // Set
    DivaCPT_RTPCodecMask,               // Set
    DivaCPT_RTPMinDejitterDelay,        // Set
    DivaCPT_RTPPacketizationInterval,   // Set
    DivaCPT_RTPMulticastTTL,            // Set

    /*
     * Calls that allow certain different speeds and framing
     * Analog calls and asynchronous V.110
     */ 
    DivaCPT_MaximumSpeed = 400,         // Set
    DivaCPT_DataBits,                   // Get Set
    DivaCPT_StopBits,                   // Get Set
    DivaCPT_Parity,                     // Get Set

    /*
     * modem parameters.
     */
    DivaCPT_DisableCompression = 800,   // Set
    DivaCPT_DisableV42,                 // Set
    DivaCPT_DisableMNP,                 // Set
    DivaCPT_ForceReliable,              // Set
    DivaCPT_DisableRetrain,             // Set
    DivaCPT_ModulationClass,            // Set

    DivaCPT_NegotiatedV42V42bis,        // Get
    DivaCPT_NegotiatedMNP4MNP5,         // Get
    DivaCPT_NegotiatedTransparent,      // Get
    DivaCPT_NegotiatedCompression,      // Get
    DivaCPT_DCD,                        // Get
    DivaCPT_CTS,                        // Get
    DivaCPT_ConnectedNorm,              // Get
    DivaCPT_RoundTripDelay,             // Get

    /*
     * Extended modem parameters
     */
    DivaCPT_GuardTone,                  // Set
    DivaCPT_ModemLeasedLine,            // Set
    DivaCPT_Modem4WireOption,           // Set
    DivaCPT_DisableDiscOnBusyTone,      // Set
    DivaCPT_DisableCallingTone,         // Set
    DivaCPT_DisableAnswerTone,          // Set
    DivaCPT_EnableDialToneDetect,       // Set
    DivaCPT_DiableStepUp,               // Set
    DivaCPT_DiableStepDown,             // Set
    DivaCPT_DisableSpiltSpeed,          // Set
    DivaCPT_EnableShortAnswerTone,      // Set
    DivaCPT_DisableFlushTimer,          // Set
    DivaCPT_EnableEmptyFrames,          // Set
    DivaCPT_EnableMultimoding,          // Set
    DivaCPT_BypassControl,              // Set
    DivaCPT_DisableModulationV21,       // Set
    DivaCPT_DisableModulationV22,       // Set
    DivaCPT_DisableModulationV22bis,    // Set
    DivaCPT_DisableModulationV23,       // Set
    DivaCPT_DisableModulationV32,       // Set
    DivaCPT_DisableModulationV32bis,    // Set
    DivaCPT_DisableModulationV34,       // Set
    DivaCPT_DisableModulationV90DPCM,   // Set
    DivaCPT_DisableModulationBell103,   // Set
    DivaCPT_DisableModulationBell212A,  // Set
    DivaCPT_DisableModulationAllFS,     // Set
    DivaCPT_DisableModulationK56Flex,   // Set
    DivaCPT_DisableModulationX2,        // Set
    DivaCPT_DisableV42Detection,        // Set
    DivaCPT_EnableModulationV29FDX,     // Set
    DivaCPT_EnableModulationV33,        // Set
    DivaCPT_EnableModulationV90APCM,    // Set
    DivaCPT_EnableModulationV22FS,      // Set
    DivaCPT_EnableModulationV29FS,      // Set
    DivaCPT_EnableModulationV23_1,      // Set
    DivaCPT_EnableModulationV23_2,      // Set
    DivaCPT_MinimumTxSpeed,             // Set
    DivaCPT_MaximumTxSpeed,             // Set
    DivaCPT_MinimumRxSpeed,             // Set
    DivaCPT_MaximumRxSpeed,             // Set
    DivaCPT_TxLevelAdjust,              // Set
    DivaCPT_DisableV34TxLevelReduction, // Set
    DivaCPT_DisableV34PreCoding,        // Set
    DivaCPT_DisableV34PreEmphasis,      // Set
    DivaCPT_DisableV34Shaping,          // Set
    DivaCPT_DisableV34NonLinearEncoding,// Set
    DivaCPT_DisableV34ManualReduction,  // Set
    DivaCPT_DisableV34Training16Point,  // Set
    DivaCPT_DisableV34SymbolRate2400,   // Set
    DivaCPT_DisableV34SymbolRate2743,   // Set
    DivaCPT_DisableV34SymbolRate2800,   // Set
    DivaCPT_DisableV34SymbolRate3000,   // Set
    DivaCPT_DisableV34SymbolRate3200,   // Set
    DivaCPT_DisableV34SymbolRate3429,   // Set
    DivaCPT_ForceReliableIfV34,         // Set
    DivaCPT_DisableSDLC,                // Set
    DivaCPT_DisableReliableIf1200,      // Set
    DivaCPT_BufferDuringV42Detection,   // Set
    DivaCPT_DisableV42SelectivReject,   // Set
    DivaCPT_DisableMNP3,                // Set
    DivaCPT_DisableMNP4,                // Set
    DivaCPT_DisableMNP10,               // Set
    DivaCPT_TransparentModeIfV22bis,    // Set
    DivaCPT_TransparentModeIfV32bis,    // Set
    DivaCPT_BreakMode,                  // Set
    DivaCPT_ModemEarlyConnect,          // Set
    DivaCPT_ModemPassIndication,        // Set
    DivaCPT_SDLCLinkAddress,            // Set
    DivaCPT_SDLCModuloMode,             // Set
    DivaCPT_SDLCWindowSize,             // Set
    DivaCPT_SDLCXID,                    // Set
    DivaCPT_SDLCReverseEstablishment,   // Set

    /*
     * V.18 parameters
     */
    DivaCPT_V18Selected,                // Set
    DivaCPT_V18ProbingSequence,         // Set
    DivaCPT_V18CountryProbingSequence,  // Set
    DivaCPT_V18ProbingMessage,          // Set
    DivaCPT_V18ReinitializeOnSilence,   // Set
    DivaCPT_V18RevertToAnswerMode,      // Set
    DivaCPT_V18DisconnectOnBusy,        // Set
    DivaCPT_V18AutomodingMonitorMode,   // Set
    DivaCPT_V18TextProbingForCarrierMode,// Set
    DivaCPT_V18TXPSpaceParityInOrigMode, // Set
    DivaCPT_V18EnableV18OriginationMode, // Set
    DivaCPT_V18EnableV18AnswerMode,     // Set
    DivaCPT_V18EnableV21OriginationMode,// Set
    DivaCPT_V18EnableV21AnswerMode,     // Set
    DivaCPT_V18EnableBell103OrigMode,   // Set
    DivaCPT_V18EnableBell103AnswerMode, // Set
    DivaCPT_V18EnableV23OriginationMode,// Set
    DivaCPT_V18EnableV23AnswerMode,     // Set
    DivaCPT_V18EnableEDTMode,           // Set
    DivaCPT_V18EnableBAUDOT45Mode,      // Set
    DivaCPT_V18EnableBAUDOT47Mode,      // Set
    DivaCPT_V18EnableBAUDOT50Mode,      // Set
    DivaCPT_V18EnableDTMFMode,          // Set
    DivaCPT_V18TransmitLevel,           // Set
    DivaCPT_V18AsyncFormatV21,          // Set
    DivaCPT_V18AsyncFormatV23,          // Set
    DivaCPT_V18AsyncFormatBell103,      // Set
    DivaCPT_V18AsyncFormatEDT,          // Set
    DivaCPT_V18AsyncFormatBAUDOT,       // Set
    DivaCPT_V18TimerTcTimeout,          // Set
    DivaCPT_V18TimerTmTimeout,          // Set
    DivaCPT_V18CleanCarrierTime,        // Set
    DivaCPT_V18EchoSupressTime,         // Set

    DivaCPT_EnableModulationV22bisFS,   // Set
    DivaCPT_SDLCInitiateFastEstablishment,// Set
    DivaCPT_SDLCSingleDataPackets,      // Set
    DivaCPT_SDLCExplicitDataAck,        // Set 
    DivaCPT_SDLCFastPollRecovery,       // Set
    DivaCPT_ModemDisconnectTimeout,     // Set

    DivaCPT_Modem7BitsPerChar,          // Set
    DivaCPT_Modem2StopBits,             // Set
    DivaCPT_ModemParityOdd,             // Set
    DivaCPT_ModemParityEven,            // Set
    DivaCPT_ModemEnableAutoMode,        // Set
    DivaCPT_ModemSDLCEnable,            // Set
    DivaCPT_ModemSyncV22Normal,         // Set
    DivaCPT_ModemSyncEnableV22bis,      // Set
    DivaCPT_ModemSyncEnableV22Fast,     // Set
    DivaCPT_ModemSyncEnableV22bisFast,  // Set
    DivaCPT_ModemSyncEnableV29Fast,     // Set
    DivaCPT_ModemSyncSDLCExtModuloMode, // Set

    DivaCPT_EnableModulationBell202CID, // Set
    DivaCPT_EnableModulationBell202POS, // Set
    DivaCPT_EnableModulationBell103SIA, // Set
    DivaCPT_EnableModulationV21Bits10,  // Set
    DivaCPT_EnableModulationV23Reverse, // Set

    DivaCPT_EnableDataStatusReporting,  // Set
    DivaCPT_AnswerToneDuration,         // Set
    DivaCPT_NegotiatedSDLC,             // Get
    DivaCPT_EnableLargeFrames,          // Set
    DivaCPT_V29FCAnswerToneDuration,    // Set
    DivaCPT_ModemBitTransparentMode,    // Set
    DivaCPT_EnableModulationFSK,        // Set
    DivaCPT_EnableFSKExtendedMode,      // Set
    DivaCPT_EnableModulationECall,      // Set

    DivaCPT_ModemDisconnectReason,      // Get
    DivaCPT_ModemStatsTxPollSequences,  // Get
    DivaCPT_ModemStatsParityErrors,     // Get
    DivaCPT_ModemStatsFramingErrors,    // Get
    DivaCPT_ModemStatsCRCErrors,        // Get
    DivaCPT_ModemStatsAbortErrors,      // Get
    DivaCPT_ModemStatsBufferOverruns,   // Get
    DivaCPT_ModemStatsTransportProtocol,// Get
    DivaCPT_ModemStatsSymbolRate,       // Get
    DivaCPT_ModemStatsSNR,              // Get
    DivaCPT_ModemStatsRxSignalLevel,    // Get
    DivaCPT_ModemStatsEchoSignalLevel,  // Get
    DivaCPT_ModemStatsSignalQuality,    // Get
    DivaCPT_FastFallbackToTransparent,  // Set
    DivaCPT_MaxGarbageBytes,            // Set
    DivaCPT_MaxTimeInDetectionPhase,    // Set
    DivaCPT_SDLCFastPollRetryTimer,     // Set    
    DivaCPT_AnswerToneDelay,            // Set    

    /*
     * Plain setting of all protocol parameters
     */
    DivaCPT_B1Protocol = 1200,          // Set
    DivaCPT_B2Protocol,                 // Set
    DivaCPT_B3Protocol,                 // Set
    DivaCPT_B1Configuration,            // Set 
    DivaCPT_B2Configuration,            // Set 
    DivaCPT_B3Configuration,            // Set 

    DivaCPT_LLC,                        // Get Set
    DivaCPT_HLC,                        // Get Set
    DivaCPT_B_ChannelInfo,              // Set
    DivaCPT_KeypadFacility,             // Get Set
    DivaCPT_UserUserInfo,               // Get Set
    DivaCPT_FacilityDataArray,          // Get Set
    DivaCPT_DisplayInfo,                // Get
    DivaCPT_TotalChargeUnits,           // Get
    DivaCPT_SpecialInfoElement,         // Get Set
    DivaCPT_ChannelInfoElement,         // Get
    DivaCPT_ProgressIndElement,         // Get
    DivaCPT_SetupMessage,               // Get
    DivaCPT_GlobalConfiguration,        // Set
    DivaCPT_ReverseDataChannelConnect,  // Set
    DivaCPT_CauseInfoElement,           // Get
    DivaCPT_SendingComplete,            // Get Set
    DivaCPT_LastMessage,                // Get
    DivaCPT_LineInfoElement,            // Get
    DivaCPT_ServiceIndicator,           // Get

    /*
     * B channel negotiation
     */
    DivaCPT_X25_NCPI = 2000,            // Get Set
    DivaCPT_X25_CalledAddress,          // Get Set
    DivaCPT_X25_CallingAddress,         // Get Set
    DivaCPT_X25_ReverseRestart,         // Set
    DivaCPT_NCPI,                       // Set

    /*
     * Autodetection
     */
    DivaCPT_AutoDetectMode = 2400,      // Set
    DivaCPT_AutoDetectX75ForceX25,      // Set
    DivaCPT_AutoDetectMaxFrames,        // Set
    DivaCPT_AutoDetectMaxSeconds,       // Set
    DivaCPT_ParityAutoDetectMode,       // Set
    DivaCPT_ParityAutoDetectMinData,    // Set
    DivaCPT_DetectedParity,             // Get Set

    /*
     * Special settings
     */
    DivaCPT_UseSameChannelForTransfer = 4000,   // Set
    DivaCPT_NoAnswerTimeout,                    // Set
    DivaCPT_ConnectTimeout,                     // Set
    DivaCPT_NoHoldBeforeTransfer,               // Set
    DivaCPT_TransferRequestNotification,        // Set
    DivaCPT_ManualDataChannel,                  // Set
    DivaCPT_AutoDiversion,                      // Set

    /*
     * Special hardware capability dependent settings
     */
    DivaCPT_UseExternalEquipment = 5000,        // Set

     /*
      * Statistics
      */
    DivaCPT_CallTimeStats = 5200,               // Get

    /*
     * Call logging / Monitoring only properties
     */
    DivaCPT_MonitorIncomingCall = 5400,          // Get

} DivaCallPropertyType;


/*
 * Properties that are expired.
 */
#define DivaCPT_DisableTrellisCoding    DivaCPT_EnableShortAnswerTone

typedef enum
{
    V18DefProbingNone = 0,
    V18DefProbingAustria,
    V18DefProbingIreland,
    V18DefProbingGermany,
    V18DefProbingSwiss,
    V18DefProbingItaly,
    V18DefProbingNetherlands,
    V18DefProbingScandinavian,
    V18DefProbingUK,
    V18DefProbingUS,
    V18DefProbingFrance,
    V18DefProbingBelgium
} DivaV18DefProbings;


typedef enum
{
    V18DataBits4            = 0x0004, 
    V18DataBits5            = 0x0005, 
    V18DataBits6            = 0x0006, 
    V18DataBits7            = 0x0007, 
    V18DataBits8            = 0x0008, 
    V18ParityNo             = 0x0000,
    V18ParityOdd            = 0x0010,
    V18ParityEven           = 0x0020,
    V18ParityMark           = 0x0030,
    V18ParitySpace          = 0x0040,
    V18EnableRxParityCheck  = 0x0080,
    V18StopBits1            = 0x0000,
    V18StopBits1_5          = 0x0010,
    V18StopBits2            = 0x0020,
} DivaV18Framing;


typedef enum
{
    ConnNormUnspecified         = 0,
    ConnNormV21                 = 1,
    ConnNormV23                 = 2,
    ConnNormV22                 = 3,
    ConnNormV22bis              = 4,
    ConnNormV32bis              = 5,
    ConnNormV34                 = 6,
    ConnNormBell212A            = 8,
    ConnNormBell103             = 9,
    ConnNormV29LeasedLine       = 10,
    ConnNormV33LeasedLine       = 11,
    ConnNormV90                 = 12,
    ConnNormV32                 = 18,
    ConnNormK56Flex             = 19,
    ConnNormX2                  = 20,

    /*
     * Text phone modulations
     */
    ConnNormTxtPhoneUnspecified = 21,
    ConnNormTxtPhoneV18Org      = 22,
    ConnNormTxtPhoneV18Ans      = 23,
    ConnNormTxtPhoneV21Org      = 24,
    ConnNormTxtPhoneV21Ans      = 25,
    ConnNormTxtPhoneBell103Org  = 26,
    ConnNormTxtPhoneBell103Ans  = 27,
    ConnNormTxtPhoneV23Org      = 28,
    ConnNormTxtPhoneV23Ans      = 29,
    ConnNormTxtPhoneEDT         = 30,
    ConnNormTxtPhoneBAUDOT45    = 31,
    ConnNormTxtPhoneBAUDOT47    = 32,
    ConnNormTxtPhoneBAUDOT50    = 33,
    ConnNormTxtPhoneDTMF        = 34,

    /*
     * Special modulations, sync modem and fast setup
     */
    ConnNormV23ETS300659_2      = 39,
    ConnNormV23ETS300659_1      = 40,
    ConnNormBell202CID          = 42,
    ConnNormV21Bits10           = 43,
    ConnNormBell202POS          = 44,
    ConnNormBell103SIA          = 45,
    ConnNormV23Reverse          = 46,
    ConnNormFSK                 = 47,
    ConnNormECall               = 59,
    ConnNormV22FastSetup        = 72,
    ConnNormV22bisFastSetup     = 73,
    ConnNormV29FastSetup        = 74,
} DivaConnectedNorm;


typedef enum
{
    DivaMDRNormal = 0,
    DivaMDRDialError,
    DivaMDRIncompatibility,
    DivaMDRTrainingTimeout,
    DivaMDRNoCarrier,
    DivaMDRCarrierLoss,
    DivaMDRFrameReject,
    DivaMDRV42bisInconsistency,
    DivaMDRApplicationAbort,
    DivaMDRFatalDSPError,
    DivaMDRTimeoutEstablishment,
    DivaMDRTimeoutPolling,
    DivaMDRRetriesTxExceeded,
    DivaMDRRetriesRxExceeded,
    DivaMDRRetransmissionsTxExceeded,
    DivaMDRRetransmissionsRxExceeded,
} DivaModemDisconnectReasons;

typedef enum
{
    DivaModemTransportNone = 0,
    DivaModemTransportPACK8,
    DivaModemTransportHDLCOverAsync,
} DivaModemTransportProtocol;

typedef struct
{
    unsigned char           MSDLength;
    unsigned char           MSD[255];
} DivaECallFrame;

typedef enum
{
    DivaECallCommandSendMSD     = 0,
    DivaECallCommandSendHLACK   = 0x0a,
};

typedef struct
{
    unsigned char           nDataLength;
    unsigned char           Data[255];
} DivaBinaryData;

typedef struct
{
    DWORD                   nDataLength;
    unsigned char           Data[2048];
} DivaSetupMessage;

typedef struct
{
    unsigned char           Number[MAX_ADDR_LEN];
} DivaPlainNumber;

typedef struct
{
    DWORD                   TypeOfNumber;
    DWORD                   NumberIdentification;
    DWORD                   Presentation;
    DWORD                   Screening;
} DivaNumberInformation;

typedef enum
{
    DivaNumberTypeUnkown                = 0x00,
    DivaNumberTypeInternational         = 0x01,
    DivaNumberTypeNational              = 0x02,
    DivaNumberTypeNetwork               = 0x03,
    DivaNumberTypeSubscriber            = 0x04,
    DivaNumberTypeAbbreviated           = 0x06,
} DivaNumberType;

typedef enum
{
    DivaNumberIdUnknown                 = 0x00,
    DivaNumberIdISDNTelephony           = 0x01,
    DivaNumberIdData                    = 0x03,
    DivaNumberIdNational                = 0x08,
    DivaNumberIdPrivate                 = 0x09
} DivaNumberId;

typedef enum
{
    DivaNumberPresentationAllowed       = 0x00,
    DivaNumberPresentationRestricted    = 0x01,
    DivaNumberPresentationNotAvailable  = 0x02
} DivaNumberPresentation;

typedef enum
{
    DivaNumberScreeningUser             = 0x00,
    DivaNumberScreeningUserPassed       = 0x01,
    DivaNumberScreeningUserFailed       = 0x02,
    DivaNumberScreeningNetwork
} DivaNumberScreening;

typedef enum
{
    DivaAutoDetectAuto                  = 0x01,
    DivaAutoDetectDigital               = 0x02,
} DivaAutoDetectModes;

typedef enum 
{
    DivaParityDetectModeDisabled        = 0x00, 
    DivaParityDetectModeTranspEven      = 0x01, 
    DivaParityDetectModeTranspOdd       = 0x02, 
    DivaParityDetectModeTranspEvenOdd   = 0x03,
} DivaParityDetectMode; 

typedef enum 
{ 
    DivaParityUnknown,
    DivaParityNone, 
    DivaParityOdd, 
    DivaParityEven, 
} DivaParity;

#define MAX_START_TONES                 20


typedef enum
{
    DivaModulationClassNone = 0,
    DivaModulationClassInClass,
    DivaModulationClassV100,
    DivaModulationClassV8,
} DivaModulationClass;

typedef enum
{
    DivaFaxPageQualityPerfect = 0,
    DivaFaxPageQualityGood,
    DivaFaxPageQualityAcceptable,
    DivaFaxPageQualityReject
} DivaFaxPageQuality;

typedef enum
{
    DivaFaxPageEndUndefined = 0,
    DivaFaxPageEndMPS,
    DivaFaxPageEndEOM,
    DivaFaxPageEndEOP
} DivaFaxPageEnd;

typedef enum
{
    DivaFaxScanLineUnknown = 0,
    DivaFaxScanLineMax215,              // ISO A4 
    DivaFaxScanLineMax255,              // ISO B4
    DivaFaxScanLineMax303,              // ISO A3
} DivaFaxScanLineMax;

typedef enum
{
    DivaFaxStorePerSession = 0,         // default 
    DivaFaxStorePerDocument,            // reserved for future versions
    DivaFaxStorePerPage,                // Only for Color fax JPEG files
} DivaFaxStoreModes;

typedef enum
{
    DivaFaxStatusTrainingResult     = 0x00000001,
    DivaFaxStatusTrainingStatistics = 0x00000002,
    DivaFaxStatusQualityReport      = 0x00000004,
    DivaFaxStatusPartialPageReport  = 0x00000008,
    DivaFaxStatusTimeoutReport      = 0x00000010,
    DivaFaxStatusResultReport       = 0x00000020,
    DivaFaxStatusDISReport          = 0x00000040,
    DivaFaxStatusDCSReport          = 0x00000080,
    DivaFaxStatusPhaseReport        = 0x00000100,
} DivaFaxStatusType;


typedef struct
{
    DWORD                   GoodBytes;
    DWORD                   ErrorBytes;
    DWORD                   Noise;
} DivaFaxTrainingStats;

typedef struct
{
    DWORD                   TotalScanLines;
    DWORD                   ErrorScanLines;
    DWORD                   ConsecutiveErrors;
    DWORD                   TotalBytes;
    DWORD                   ErrorBytes;
    DWORD                   ConsecutiveErrorBytes;
} DivaFaxPageQualityDetails;

typedef struct
{
    DWORD                   ECMFrameLength;
    BYTE                    ECMState[32];
    BYTE                    PPSFrameLength;
    BYTE                    PPSFrame[255];
} DivaFaxPartialPageDetails;


typedef enum
{
    DivaFaxPhaseUnknown = 0,
    DivaFaxPhaseA = 1,
    DivaFaxPhaseB,
    DivaFaxPhaseC,
    DivaFaxPhaseD,
    DivaFaxPhaseE,
} DivaFaxPhase;

typedef enum
{
    DivaThresholdMin = -127,
    DivaThresholdMax = 127,
} DivaThreshold;

/* 
 * DivaCallPropertyValue
 */
typedef union
{
    /*
     * Common call properties for all types of calls
     */
    DWORD                   CallType;
    DWORD                   LineDevice;
    DWORD                   SignaledService;
    DivaBinaryData          BearerCapabilities;
    DivaPlainNumber         CalledNumber;
    DivaPlainNumber         CallingNumber;
    DivaNumberInformation   CalledNumberParams;
    DivaNumberInformation   CallingNumberParams;
    DivaPlainNumber         RedirectingNumber;
    DivaPlainNumber         RedirectedNumber;
    DWORD                   TxSpeed;
    DWORD                   RxSpeed;

    DWORD                   DiscReason;
    DWORD                   SignaledLineDiscReason;
    DWORD                   RejectReason;

    /*
     * Voice parameter
     */
    BOOL                    VoiceEchoCanceler;
    BOOL                    VoiceEarlyDataChannel;
    DWORD                   VoiceRecordSilenceTimeout;

    /*
     * 2nd calling party number for SMS
     */
    DivaPlainNumber         SecondCallingNumber;
    DivaNumberInformation   SecondCallingNumberParams;

    /*
     * Calling / connected name
     */
    unsigned char           CallingName[MAX_ADDR_LEN];
    unsigned char           ConnectedName[MAX_ADDR_LEN];

    /*
     * Calling / called sub address
     */
    unsigned char           CallingSubAddress[MAX_SUBADDR_LEN];
    unsigned char           CalledSubAddress[MAX_SUBADDR_LEN];
    unsigned char           OriginalCalledNumber[MAX_SUBADDR_LEN];
    unsigned char           ConnectedNumber[MAX_SUBADDR_LEN];
    unsigned char           CalledName[MAX_SUBADDR_LEN];
    DWORD                   DataChannel;

    /*
     * Disconnect reason and disconnect cause
     */
    DivaActiveDiscReasons   DisconnectReason;
    DWORD                   DisconnectCause;

    unsigned char           RedirectionNumber[MAX_SUBADDR_LEN];
    DWORD                   RedirectReason;

    /*
     * Streaming properties.
     */
    unsigned char           VoiceRecordStartTones[MAX_START_TONES];
    DWORD                   VoiceDTMF_SendDuration;
    DWORD                   VoiceDTMF_SendPause;
    DWORD                   VoiceDTMF_DetectDuration;
    DWORD                   VoiceDTMF_DetectPause;
    BOOL                    VoiceRemoveDTMFFromStream;
    BOOL                    VoiceEarlyDataDiscOnInfo;
    BOOL                    EchoCancellerEnableNLP;
    BOOL                    EchoCancellerAutoDisable1;
    BOOL                    EchoCancellerAutoDisable2;
    DWORD                   EchoCancellerTailLength;
    DWORD                   EchoCancellerPreDelay;
    BOOL                    EnableDTMFTrailingEdge;
    DivaDataCodec           DataCodec;
    DivaDataCodecOptions    DataCodecOptions;
    DWORD                   DataCodecSamplesPerPacket;
    BOOL                    DisableMFDetection;
    BOOL                    EnableHookSignaling;
    BOOL                    EnableR2ForwardDetection;
    BOOL                    EnableR2BackwardDetection;
    DivaBinaryData          FSKData;
    DWORD                   DTMFMode;
    BOOL                    EnableTransparentLI;
    int                     HumanTalkerThreshold;
    int                     VoiceActivityThreshold;
    int                     VoiceDTMF_TxLevelGroup;
    DWORD                   SingleToneOffDuration;
    DWORD                   DualToneOffDuration;
    DWORD                   HookSignalingOffhookTime;
    DWORD                   HookSignalingHookFlashTime;
    DWORD                   ToneDetectorFFTLength;
    DWORD                   SingleToneDetectorMinFreq;
    DWORD                   SingleToneDetectorMaxFreq;
    DivaSampleRates         DataCodecSampleRate;

    /*
     * Fax parameter
     */
    unsigned char           FaxLocalId[FAX_PARAM_LENGTH];
    unsigned char           FaxHeadline[1024];
    unsigned char           FaxRemoteId[FAX_PARAM_LENGTH];
    DWORD                   FaxPages;
    DWORD                   FaxMaxSpeed;
    BOOL                    FaxHighResolution;
    BOOL                    FaxEnablePolling;
    BOOL                    FaxReverseSession;
    BOOL                    FaxMultiDocument;
    BOOL                    FaxDisableECM;
    BOOL                    FaxDisableMR;
    BOOL                    FaxDisableMMR;
    DivaFaxPageQuality      FaxPageQuality;
    DivaFaxPageEnd          FaxPageEndInfo;
    unsigned char           FaxRemoteFeatures[20];
    DWORD                   FaxRemoteMaxHorzRes;
    DWORD                   FaxRemoteMaxVertRes;
    DWORD                   FaxRemoteMaxSpeed;
    unsigned char           FaxRemoteNSF[MAX_SUBADDR_LEN];
    unsigned char           FaxLocalNSF[MAX_SUBADDR_LEN];
    BOOL                    FaxEnableColor;
    BOOL                    FaxColorSelected;
    BOOL                    EnableInterrupt;
    BOOL                    RequestInterrupt;
    BOOL                    FaxProcedureInterrupt;
    BOOL                    FaxEnableSecurity;
    BOOL                    FaxRemoteSupportsSubaddr;
    BOOL                    FaxRemoteSupportsPassword;
    unsigned char           FaxSignalSubAddress[FAX_PARAM_LENGTH];
    unsigned char           FaxSignalPassword[FAX_PARAM_LENGTH];
    unsigned char           FaxRemoteSubAddress[FAX_PARAM_LENGTH];
    unsigned char           FaxRemotePassword[FAX_PARAM_LENGTH];
    BOOL                    FaxDisableFileBuffering;
    BOOL                    FaxUseTextForSending;
    BOOL                    FaxAllowDocumentStretching;
    DivaFaxScanLineMax      FaxRemoteScanLineLength;
    DWORD                   FaxStoreMode;
    BOOL                    FaxPassDataOnNextPageStart;
    DWORD                   FaxStartPage;
    BOOL                    FaxEnableClearChannel;
    BOOL                    EnableFaxStatusReporting;
    BYTE                    FaxReportDCS[20];
    BOOL                    FaxReportTrainingResult;
    DivaFaxTrainingStats    FaxReportTrainingStats;
    DivaFaxPageQualityDetails   FaxReportPageQuality;
    DivaFaxPartialPageDetails   FaxReportPartialPage;
    DWORD                   FaxReportT30Timeout;
    DivaFaxPhase            FaxT30Phase;
    DWORD                   FaxResultReport;
    BOOL                    FaxEnableBinaryFileTransfer;
    BOOL                    FaxRequestBinaryFileTransfer;
    BOOL                    FaxBinaryFileTransferActive;
    BOOL                    FaxSelectManualResult;
    DWORD                   FaxManualResultValue;
    BOOL                    FaxUseHeadlineFont;
    unsigned char           FaxHeadlinePageInfo[1024];
    unsigned char           FaxHeadlineFontFile[2048];
    DWORD                   FaxHeadlineFontFace;
    DWORD                   FaxHeadlineFontSizePoints;
    DivaFaxHeadlineCoding   FaxHeadlineCoding;
    BOOL                    FaxHeadlineOmitDateTime;
    BOOL                    FaxHeadlineOmitPageInfo;

    /*
     * VoIP / RTP Parameter
     */
    DivaPayloadProtocol     RTPPayloadProtocol;
    DWORD                   RTPPayloadOptions;
    DWORD                   RTPMaxPacketLateRate;
    DWORD                   RTPMaxDejitterDelay;
    DWORD                   RTPSSRC;
    char                    RTPLocalIpAddress[IP_ADDRESS_LENGTH];
    DWORD                   RTPLocalPort;
    char                    RTPRemoteIpAddress[IP_ADDRESS_LENGTH];
    DWORD                   RTPRemotePort;
    DWORD                   RTPCodecMask;
    DWORD                   RTPMinDejitterDelay;
    DWORD                   RTPPacketizationInterval;
    DWORD                   RTPMulticastTTL;

    /*
     * Calls that allow certain different speeds and framing
     * Analog calls and asynchronous V.110
     */
    DWORD                   MaximumSpeed;
    DWORD                   DataBits;
    DWORD                   StopBits;
    DWORD                   Parity;

    /*
     * modem parameters.
     */
    BOOL                    DisableCompression;
    BOOL                    DisableV42;
    BOOL                    DisableMNP;
    BOOL                    ForceReliable;
    BOOL                    DisableRetrain;
    DivaModulationClass     ModulationClass;

    BOOL                    NegotiatedV42V42bis;
    BOOL                    NegotiatedMNP4MNP5;
    BOOL                    NegotiatedTransparent;
    BOOL                    NegotiatedCompression;
    BOOL                    DCD;
    BOOL                    CTS;
    DWORD                   ConnectedNorm;
    DWORD                   RoundTripDelay;

    /*
     * Extended modem parameters
     */
    BOOL                    GuardTone;
    BOOL                    ModemLeasedLine;
    BOOL                    Modem4WireOption;
    BOOL                    DisableDiscOnBusyTone;
    BOOL                    DisableCallingTone;
    BOOL                    DisableAnswerTone;
    BOOL                    EnableDialToneDetect;
    BOOL                    DiableStepUp;
    BOOL                    DiableStepDown;
    BOOL                    DisableSpiltSpeed;
    BOOL                    DisableTrellisCoding;
    BOOL                    DisableFlushTimer;
    BOOL                    EnableEmptyFrames;
    BOOL                    EnableMultimoding;
    BOOL                    BypassControl;
    BOOL                    DisableModulationV21;
    BOOL                    DisableModulationV22;
    BOOL                    DisableModulationV22bis;
    BOOL                    DisableModulationV23;
    BOOL                    DisableModulationV32;
    BOOL                    DisableModulationV32bis;
    BOOL                    DisableModulationV34;
    BOOL                    DisableModulationV90DPCM;
    BOOL                    DisableModulationBell103;
    BOOL                    DisableModulationBell212A;
    BOOL                    DisableModulationAllFS;
    BOOL                    DisableModulationK56Flex;
    BOOL                    DisableModulationX2;
    BOOL                    DisableV42Detection;
    BOOL                    EnableModulationV29FDX;
    BOOL                    EnableModulationV33;
    BOOL                    EnableModulationV90APCM;
    BOOL                    EnableModulationV22FS;
    BOOL                    EnableModulationV29FS;
    BOOL                    EnableModulationV23_1;
    BOOL                    EnableModulationV23_2;
    DWORD                   MinimumTxSpeed;
    DWORD                   MaximumTxSpeed;
    DWORD                   MinimumRxSpeed;
    DWORD                   MaximumRxSpeed;
    DWORD                   TxLevelAdjust;
    BOOL                    DisableV34TxLevelReduction;
    BOOL                    DisableV34PreCoding;
    BOOL                    DisableV34PreEmphasis;
    BOOL                    DisableV34Shaping;
    BOOL                    DisableV34NonLinearEncoding;
    BOOL                    DisableV34ManualReduction;
    BOOL                    DisableV34Training16Point;
    BOOL                    DisableV34SymbolRate2400;
    BOOL                    DisableV34SymbolRate2743;
    BOOL                    DisableV34SymbolRate2800;
    BOOL                    DisableV34SymbolRate3000;
    BOOL                    DisableV34SymbolRate3200;
    BOOL                    DisableV34SymbolRate3429;
    BOOL                    ForceReliableIfV34;
    BOOL                    DisableSDLC;
    BOOL                    FastFallbackToTransparent;
    BOOL                    DisableReliableIf1200;
    BOOL                    BufferDuringV42Detection;
    BOOL                    DisableV42SelectivReject;
    BOOL                    DisableMNP3;
    BOOL                    DisableMNP4;
    BOOL                    DisableMNP10;
    BOOL                    TransparentModeIfV22bis;
    BOOL                    TransparentModeIfV32bis;
    BOOL                    BreakMode;
    BYTE                    MaxGarbageBytes;
    WORD                    MaxTimeInDetectionPhase;
    BOOL                    ModemEarlyConnect;
    BOOL                    ModemPassIndication;
    DWORD                   SDLCLinkAddress;
    DWORD                   SDLCModuloMode;
    DWORD                   SDLCWindowSize;
    BOOL                    SDLCXID;
    BOOL                    SDLCReverseEstablishment;
    BOOL                    SDLCFastPollRetryTimer;

    /*
     * V.18 parameters
     */
    BOOL                    V18Selected;
    DivaBinaryData          V18ProbingSequence;
    BOOL                    V18ProbingMessage;
    BOOL                    V18ReinitializeOnSilence;
    BOOL                    V18RevertToAnswerMode;
    BOOL                    V18DisconnectOnBusy;
    BOOL                    V18AutomodingMonitorMode;
    BOOL                    V18TextProbingForCarrierMode;
    BOOL                    V18TXPSpaceParityInOrigMode;
    BOOL                    V18EnableV18OriginationMode;
    BOOL                    V18EnableV18AnswerMode;
    BOOL                    V18EnableV21OriginationMode;
    BOOL                    V18EnableV21AnswerMode;
    BOOL                    V18EnableBell103OrigMode;
    BOOL                    V18EnableBell103AnswerMode;
    BOOL                    V18EnableV23OriginationMode;
    BOOL                    V18EnableV23AnswerMode;
    BOOL                    V18EnableEDTMode;
    BOOL                    V18EnableBAUDOT45Mode;
    BOOL                    V18EnableBAUDOT47Mode;
    BOOL                    V18EnableBAUDOT50Mode;
    BOOL                    V18EnableDTMFMode;
    DWORD                   V18TransmitLevel;
    DWORD                   V18AsyncFormatV21;
    DWORD                   V18AsyncFormatV23;
    DWORD                   V18AsyncFormatBell103;
    DWORD                   V18AsyncFormatEDT;
    DWORD                   V18AsyncFormatBAUDOT;
    DWORD                   V18TimerTcTimeout;
    DWORD                   V18TimerTmTimeout;
    DWORD                   V18CleanCarrierTime;
    DWORD                   V18EchoSupressTime;
    BOOL                    EnableModulationV22bisFS;
    BOOL                    SDLCInitiateFastEstablishment;
    BOOL                    SDLCSingleDataPackets;
    BOOL                    SDLCExplicitDataAck; 
    BOOL                    SDLCFastPollRecovery;
    DWORD                   ModemDisconnectTimeout;
    BOOL                    EnableModulationBell202CID;
    BOOL                    EnableModulationBell202POS;
    BOOL                    EnableModulationBell103SIA;
    BOOL                    EnableModulationV21Bits10;
    BOOL                    EnableModulationV23Reverse;
    BOOL                    EnableDataStatusReporting;
    DWORD                   AnswerToneDuration;
    BOOL                    NegotiatedSDLC;
    BOOL                    EnableLargeFrames;
    DWORD                   V29FCAnswerToneDuration;
    DWORD                   AnswerToneDelay;
    BOOL                    ModemBitTransparentMode;
    BOOL                    EnableModulationFSK;
    BOOL                    EnableFSKExtendedMode;
    BOOL                    EnableModulationECall;

    DivaModemDisconnectReasons  ModemDisconnectReason;
    DWORD                   ModemStatsTxPollSequences;
    DWORD                   ModemStatsParityErrors;
    DWORD                   ModemStatsFramingErrors;
    DWORD                   ModemStatsCRCErrors;
    DWORD                   ModemStatsAbortErrors;
    DWORD                   ModemStatsBufferOverruns;
    DivaModemTransportProtocol  ModemStatsTransportProtocol;
    DWORD                   ModemStatsSymbolRate;
    DWORD                   ModemStatsSNR;
    DWORD                   ModemStatsRxSignalLevel;
    DWORD                   ModemStatsEchoSignalLevel;
    DWORD                   ModemStatsSignalQuality;

    /*
     * Plain setting of all protocol parameters
     */
    DWORD                   B1Protocol;
    DWORD                   B2Protocol;
    DWORD                   B3Protocol;
    DivaBinaryData          B1Configuration; 
    DivaBinaryData          B2Configuration; 
    DivaBinaryData          B3Configuration;
    DivaBinaryData          LLC;
    DivaBinaryData          HLC;
    DivaBinaryData          B_ChannelInfo;
    DivaBinaryData          KeypadFacility;
    DivaBinaryData          UserUserInfo;
    DivaBinaryData          FacilityDataArray;
    DivaBinaryData          DisplayInfo;
    DWORD                   TotalChargeUnits;
    DivaBinaryData          SpecialInfoElement;
    DivaBinaryData          ChannelInfoElement;
    DivaBinaryData          ProgressInfoElement;
    DivaSetupMessage        SetupMessage;
    DWORD                   GlobalConfiguration;
    BOOL                    ReverseDataChannelConnect;
    DivaBinaryData          CauseInfoElement;
    BOOL                    SendingComplete;
    DivaSetupMessage        LastMessage;
    DivaBinaryData          LineInfoElement;
    DWORD                   ServiceIndicator;

    /*
     * B channel negotiation
     */
    DivaBinaryData          X25_NCPI;
    unsigned char           X25_CalledAddress[X25_ADDRESS_LENGTH];
    unsigned char           X25_CallingAddress[X25_ADDRESS_LENGTH];
    BOOL                    X25_ReverseRestart;
    DivaBinaryData          NCPI;

    /*
     * Autodetection
     */
    DWORD                   AutoDetectMode;
    BOOL                    AutoDetectX75ForceX25;
    DWORD                   AutoDetectMaxFrames;
    DWORD                   AutoDetectMaxSeconds;
    DWORD                   ParityAutoDetectMode;
    DWORD                   ParityAutoDetectMinData;
    DWORD                   DetectedParity;

    /*
     * Special settings
     */
    BOOL                    UseSameChannelForTransfer;
    DWORD                   NoAnswerTimeout;
    DWORD                   ConnectTimeout;
    BOOL                    NoHoldBeforeTransfer;
    BOOL                    TransferRequestNotification;
    BOOL                    ManualDataChannel;
    BOOL                    AutoDiversion;

    /*
     * Special hardware capability dependent settings
     */
    BOOL                    UseExternalEquipment;

    /*
     * Statistics
     */
    DivaCallTimeStatistics  CallTimeStats;

    /*
     * Call logging / Monitoring only properties
     */
    BOOL                    MonitorIncomingCall;


} DivaCallPropertyValue;


DIVA_SERVER_API DWORD DivaSetCallProperties (
    DivaCallHandle          hCall,
    DivaCallPropertyType    Type,
    DivaCallPropertyValue   *pPropertyValue,
    DWORD                   PropertyValueSize
    );

DIVA_SERVER_API DWORD DivaGetCallProperties (
    DivaCallHandle          hCall,
    DivaCallPropertyType    Type,
    DivaCallPropertyValue   *pPropertyValue,
    DWORD                   PropertyValueSize
    );

DIVA_SERVER_API DWORD DivaDefaultCallProperties (
    DivaCallHandle          hCall,
    DivaCallType            Type 
    );

/*
 * Application controled logging
 */
typedef enum
{
    DivaTraceLevelNothing = 0,
    DivaTraceLevelError,
    DivaTraceLevelWarning,
    DivaTraceLevelInformation,
    DivaTraceLevelApiEntry,
    DivaTraceLevelDebug,
    DivaTraceLevelDebugHigh,
    DivaTraceLevelApiData,
    DivaTraceLevelDataMsg,
} DivaTraceLevel;


DIVA_SERVER_API BOOL DivaEnableTrace (
    DWORD               nLevel
    );

DIVA_SERVER_API BOOL DivaSetTraceFile (
    char*               pFilename,
    DWORD               dwMaxSize
    );

DIVA_SERVER_API BOOL DivaSetTraceFileEx (
    char*               strFilename,
    DWORD               dwMaxSize,
    DWORD               dwMaxFiles,
    DWORD               dwMaxErrorFiles
    );

/*
 * Monitoring of D and B Channel
 */
typedef enum
{
    DivaMonitorSourceUnknown = 0,
    DivaMonitorSourceOriginator = 1,
    DivaMonitorSourceAnswerer,
    DivaMonitorDeviceAToDeviceB,
    DivaMonitorDeviceBToDeviceA,
    DivaMonitorSourceBoth,
    DivaMonitorSourceMixed,
} DivaMonitorSource;


typedef enum
{
    DivaMonitorStarted = 1,
    DivaMonitorStopped,
    DivaMonitorErrorStarting,
    DivaMonitorLayer1Up,
    DivaMonitorLayer1Down,
} DivaMonitorStatus;

typedef enum
{
    DivaMonitorR2Basic = 1,
    DivaMonitorR2India,
    DivaMonitorR2Brazil,
    DivaMonitorR2Mexico,
} DivaMonitorR2Variants;


typedef enum
{
    DivaMonitorOptionPolarity                   = 0x0001,
    DivaMonitorOptionConnectOnEnergy            = 0x0002,
    DivaMonitorOptionDisableCPAOnConnect        = 0x0004,
    DivaMonitorOptionDisableCPAOnDisconnect     = 0x0008,
    DivaMonitorOptionDisableStandardTones       = 0x0010,
    DivaMonitorOptionDisableFSK                 = 0x0020,
    DivaMonitorOptionDisableCPA                 = 0x0080,
    DivaMonitorOptionDisableDigitProcessing     = 0x0100,
} DivaMonitorOptions;

typedef struct
{
    DWORD               Size;
    int                 RecordingGain;
    int                 ConnectEnergy;
    DWORD               ConnectSilenceTimeout;
    DivaAudioFormat     DefaultAudioFormat;
} DivaMonitorAnalogParams;

typedef enum
{
    DivaMonitorT1CASLoopStart = 1,
    DivaMonitorT1CASGroundStart,
    DivaMonitorT1CASWinkStart,
} DivaMonitorT1CASVariants;

typedef struct
{
    DWORD           Size;
    int             RecordingGain;
    DWORD           ConnectSilenceTimeout;
    DWORD           DigitCountToSignalCall;
} DivaMonitorT1CASParams;

typedef enum
{
    DivaMonitorFrameReportModeNone,
    DivaMonitorFrameReportModeMessage,
    DivaMonitorFrameReportModeLayer3,
    DivaMonitorFrameReportModeLayer2,
} DivaMonitorFrameReportMode;

typedef struct
{
    char                Digit;
    DWORD               Device;
    DivaMonitorSource   Source;
} DivaMonitorDTMFInfo;

typedef struct
{
    DWORD               Tone;
    DWORD               Device;
    DivaMonitorSource   Source;
} DivaMonitorToneInfo;

DIVA_SERVER_API DWORD   DivaCreateMonitor ( 
    DivaAppHandle       hApp,
    AppMonitorHandle    haMonitor,
    DivaMonitorHandle   *phdMonitor,
    DWORD               LineDeviceA,
    DWORD               LineDeviceB 
    );

DIVA_SERVER_API DWORD   DivaCreateMonitorR2 ( 
    DivaAppHandle       hApp,
    AppMonitorHandle    haMonitor,
    DivaMonitorHandle   *phdMonitor,
    DWORD               LineDeviceA,
    DWORD               LineDeviceB,
    DWORD               R2Variant,
    void*               R2GenericParams,
    DWORD               Reserved
    );

DIVA_SERVER_API DWORD   DivaCreateMonitorAnalog ( 
    DivaAppHandle               hApp,
    AppMonitorHandle            haMonitor,
    DivaMonitorHandle           *phdMonitor,
    DWORD                       LineDevice,
    DWORD                       Options,
    DivaMonitorAnalogParams*    pParams
    );

DIVA_SERVER_API DWORD   DivaCreateMonitorT1CAS ( 
    DivaAppHandle               hApp,
    AppMonitorHandle            haMonitor,
    DivaMonitorHandle           *phdMonitor,
    DWORD                       LineDeviceA,
    DWORD                       LineDeviceB,
    DWORD                       T1CASVariant,
    DWORD                       Options,
    DivaMonitorT1CASParams*     pParams );

DIVA_SERVER_API DWORD   DivaCreateMonitorAudio ( 
    DivaAppHandle       hApp,
    AppMonitorHandle    haMonitor,
    DivaMonitorHandle   *phdMonitor,
    DWORD               LineDeviceA,
    DWORD               LineDeviceB 
    );

DIVA_SERVER_API DWORD   DivaDestroyMonitor (
    DivaMonitorHandle   hdMonitor
    );

DIVA_SERVER_API DWORD   DivaMonitorAttachToLine (
    DivaMonitorHandle   hdMonitor,
    DWORD               Line,
    DivaCallHandle      *phdCall
    );

DIVA_SERVER_API DWORD   DivaMonitorAttachToTimeslot (
    DivaMonitorHandle   hdMonitor,
    DWORD               Timeslot,
    DivaCallHandle      *phdCall
    );

DIVA_SERVER_API DWORD   DivaMonitorDetachHandle (
    DivaMonitorHandle   hdMonitor,
    DivaCallHandle      hdCall
    );

DIVA_SERVER_API DWORD   DivaMonitorRecordAudio (
    DivaMonitorHandle   hdMonitor,
    DivaCallHandle      hCall,
    char                *pFilename,
    DivaAudioFormat     Format,
    DivaMonitorSource   Source 
    );

DIVA_SERVER_API DWORD   DivaMonitorEnableAudioData (
    DivaMonitorHandle   hdMonitor,
    DivaCallHandle      hCall,
    DWORD               AudioBuffers
    );

DIVA_SERVER_API DWORD   DivaMonitorDisableAudioData (
    DivaMonitorHandle   hdMonitor,
    DivaCallHandle      hCall
    );

DIVA_SERVER_API DWORD   DivaMonitorReceiveAudio (
    DivaMonitorHandle   hdMonitor,
    DivaCallHandle      hCall,
    DivaMonitorSource   Source,
    unsigned char*      pBuffer,
    DWORD               BufferSize,
    DWORD*              pBytesWritten,
    DivaAudioFormat     Format
    );

DIVA_SERVER_API DWORD   DivaMonitorStopAudio (
    DivaMonitorHandle   hdMonitor,
    DivaCallHandle      hCall,
    DivaMonitorSource   Source 
    );
DIVA_SERVER_API DWORD   DivaMonitorSetVolume (
    DivaMonitorHandle   hdMonitor,
    DivaCallHandle      hCall,
    DivaVolume          Volume,
    DivaMonitorSource   Source 
    );

DIVA_SERVER_API DWORD DivaMonitorGetCallInfo (
    DivaMonitorHandle   hdMonitor,
    DivaCallHandle      hdCall,
    DivaCallInfo*       pCallInfo
    );

DIVA_SERVER_API DWORD DivaMonitorGetCallProperties (
    DivaMonitorHandle           hdMonitor,
    DivaCallHandle              hCall,
    DivaCallPropertyType        Type,
    DivaCallPropertyValue       *pPropertyValue,
    DWORD                       PropertyValueSize
    );

DIVA_SERVER_API DWORD DivaMonitorGetSetupMessage (
    DivaMonitorHandle           hdMonitor,
    DivaCallHandle              hdCall,
    unsigned char*              pBuffer,
    DWORD                       BufferLength,
    DWORD*                      pBytesUsed               
    );

DIVA_SERVER_API DWORD DivaMonitorCloseCallHandle (
    DivaMonitorHandle           hdMonitor,
    DivaCallHandle              hdCall
    );

DIVA_SERVER_API DWORD DivaMonitorReportFrames (
    DivaMonitorHandle           hdMonitor,
    DivaMonitorFrameReportMode  Mode
    );

DIVA_SERVER_API DWORD DivaMonitorGetFrame (
    DivaMonitorHandle           hdMonitor,
    DivaHandle                  hFrame,
    DivaCallHandle*             phdCall,
    DWORD*                      pLineDevice,
    DivaTime*                   pTimeStamp,
    unsigned char*              pBuffer,
    DWORD                       BufferLength,
    DWORD*                      pBytesUsed               
    );

DIVA_SERVER_API DWORD DivaMonitorSpecifyTone (
    DivaMonitorHandle           hdMonitor,
    DWORD                       Type,
    DWORD                       Recurrences,
    DWORD                       NumDefinitions,
    DivaToneDefinition*         pDefinitions
    );

DIVA_SERVER_API DWORD DivaMonitorReportDTMF ( 
    DivaMonitorHandle           hdMonitor,
    DivaCallHandle              hCall,
    BOOL                        bEnable
    );

DIVA_SERVER_API DWORD DivaMonitorReportTone ( 
    DivaMonitorHandle           hdMonitor,
    DivaCallHandle              hCall,
    BOOL                        bEnable
    );

DIVA_SERVER_API DWORD DivaMonitorGetDTMFInfo ( 
    DivaMonitorHandle           hdMonitor,
    DivaCallHandle              hCall,
    DivaMonitorDTMFInfo*        pInfo
    );

DIVA_SERVER_API DWORD DivaMonitorGetToneInfo (
    DivaMonitorHandle           hdMonitor,
    DivaCallHandle              hCall,
    DivaMonitorToneInfo*        pInfo
    );


/*
 * Plain audio streaming registration (ASR TTS engine interface)
 */
typedef enum
{
    DivaIdFormatDWORD,      // .Id contains a DWORD
    DivaIdFormatString,     // .Id contains a zero terminated string
    DivaIdFormatBinary,     // First byte of .Id is length followed by binary data
    DivaIdFormatNone = 0x80000000,
} DivaIdFormat;

typedef struct
{
    DWORD               IdFormat;
    unsigned char       Id[100];
} DivaIdDescriptor;


/*
 * Entries provided by the SDK.
 */
typedef DWORD (*DivaAPCloseAudio) (
    DivaAppHandle               hApp,
    DivaCallHandle              hdCall
    );

typedef DWORD (*DivaAPSendAudio) (
    DivaAppHandle               hApp,
    DivaCallHandle              hdCall,
    unsigned char*              pData,
    DWORD                       Length,
    DivaAudioFormat             Format
    );

typedef DWORD (*DivaAPStopSendAudio) (
    DivaAppHandle               hApp,
    DivaCallHandle              hdCall
    );

typedef void (*DivaAPPassEvent) (
    DivaAppHandle               hApp,
    DivaCallHandle              hdCall,
    DWORD                       Event,
    void*                       EventContext
    );

typedef void (*DivaAPSetRecordFormat) (
    DivaAppHandle               hApp,
    DivaCallHandle              hdCall,
    DivaAudioFormat             Format
    );

typedef void (*DivaAPSetVolume) (
    DivaAppHandle               hApp,
    DivaCallHandle              hdCall,
    DivaVolume                  Volume,
    DivaDirection               Direction
    );

/*
 * Entries provided by the audio provider.
 */
typedef void (*DivaAPNotifyCallClose) (
    AppCallHandle               hAPCall
    );

typedef void (*DivaAPNotifyReceiveAudio) (
    AppCallHandle               hAPCall,
    unsigned char*              pData,
    DWORD                       Length
    );

typedef void (*DivaAPConfirmAudioSend) (
    AppCallHandle               hAPCall
    );

typedef void (*DivaAPGetEventDetails) (
    AppCallHandle               hAPCall,
    void*                       EventContext,
    DWORD                       EventType,
    void*                       EventBuffer,
    DWORD                       EventBufferSize,
    DWORD*                      pBytesWritten
    );

typedef struct
{
    DivaCallHandle              hdCall;
    DivaIdDescriptor            Identifier;
    DivaAPSendAudio             pfnSendAudio;
    DivaAPStopSendAudio         pfnStopSendAudio;
    DivaAPSetRecordFormat       pfnSetRecordAudio;
    DivaAPCloseAudio            pfnCloseAudio;
    DivaAPPassEvent             pfnPassEvent;       // Reserved, must be NULL
    DivaAPSetVolume             pfnSetVolume;
} DivaAPNotifyCallInParams;

typedef struct
{
    AppCallHandle               hAPCall;
    DivaAPNotifyCallClose       pfnNotifyCallClose;
    DivaAPNotifyReceiveAudio    pfnNotifyReceiveAudio;
    DivaAPConfirmAudioSend      pfnConfirmSend;
    DivaAPGetEventDetails       pfnGetEventDetails; // Reserved, must be NULL
    DivaAudioFormat             Format;
} DivaAPNotifyCallOutParams;

typedef BOOL (*DivaAPNotifyCall) (
    DivaAppHandle               hApp,
    DivaAPNotifyCallInParams*   pInParams,
    DivaAPNotifyCallOutParams*  pOutParams
    );

DIVA_SERVER_API DWORD DivaRegisterAudioProvider ( 
    DivaAppHandle               *pHandle,
    char*                       pProviderName,
    DivaAPNotifyCall            pfnNotifyCall
    );

DIVA_SERVER_API DWORD DivaReleaseAudioProvider ( 
    DivaAppHandle               hApp
    );

/*
 * Assignment of audio channels by the application.
 */
typedef enum
{
    DivaAPModeAudioReceive          = 0x01,
    DivaAPModeAudioSend             = 0x10,
} DivaAPMode;

DIVA_SERVER_API DWORD DivaConnectAudioProvider (
    DivaCallHandle              hSDKCall, 
    char*                       Providername, 
    DivaIdDescriptor*           pDeviceId,
    DivaAPMode                  WhatToConnect );

DIVA_SERVER_API DWORD DivaDisconnectAudioProvider (
    DivaCallHandle              hSDKCall, 
    char*                       Providername, 
    DivaAPMode                  WhatToDisconnect );


/*
 * Message Waiting / Supplementary Service Extensions
 */
typedef enum
{
    DivaMessageAdded = 0,
    DivaMessageRemoved = 1,
    DivaMessageUnknown = 0xffff,
} DivaMessageStatus;

typedef enum
{
    DivaMessageNumberUnknown = 0xfffffffe,
    DivaMessageNumberNotAvailable =  0xffffffff
} DivaMessageNumberInfo;

typedef enum
{
    DivaMessageInvokeDeferred = 0,
    DivaMessageInvokeImmediate,
    DivaMessageInvokeCombined,
    DivaMessageInvokeSupress
} DivaMessageInvokeMode;

typedef struct
{
    DWORD                       Size;

    DWORD                       Handle;
    DWORD                       Service;
    DWORD                       NumMessages;
    DivaMessageStatus           Status;
    DWORD                       Reference;
    DivaMessageInvokeMode       InvokeMode;
    char                        ReceivingUserNumber[MAX_ADDR_LEN];
    char                        ControllingUserNumber[MAX_ADDR_LEN];
    char                        ControllingUserProvidedNumber[MAX_ADDR_LEN];
    DivaTime                    Time;
} DivaMWIActivateParams;

typedef struct
{
    DWORD                       Size;

    DWORD                       Handle;
    DWORD                       Service;
    DivaMessageInvokeMode       InvokeMode;
    char                        ReceivingUserNumber[MAX_ADDR_LEN];
    char                        ControllingUserNumber[MAX_ADDR_LEN];
} DivaMWIDeactivateParams;

typedef struct
{
    DWORD                       SizeUsed;
    DWORD                       Line;

    DWORD                       Handle;
    DWORD                       Service;
    DWORD                       NumMessages;
    DivaMessageStatus           Status;
    DWORD                       Reference;
    char                        ControllingUserNumber[MAX_ADDR_LEN];
    char                        ControllingUserProvidedNumber[MAX_ADDR_LEN];
    DivaTime                    Time;
    char                        CalledNumber[MAX_ADDR_LEN];
} DivaMWIIndicationParams;


DIVA_SERVER_API DWORD DivaMWIActivate ( 
    DivaAppHandle               hApp,
    DWORD                       LineDevice,
    DivaMWIActivateParams*      pParams
    );

DIVA_SERVER_API DWORD DivaMWIDeactivate ( 
    DivaAppHandle               hApp,
    DWORD                       LineDevice,
    DivaMWIDeactivateParams*      pParams
    );

DIVA_SERVER_API DWORD DivaMWIReport ( 
    DivaAppHandle               hApp,
    DWORD                       LineDevice,
    BOOL                        bEnable 
    );

DIVA_SERVER_API DWORD DivaMWIGetIndication ( 
    DivaAppHandle               hApp,
    DWORD                       LineDevice,
    DivaHandle                  hMWIIndication,
    DivaMWIIndicationParams*    pParams,
    DWORD                       ParamSize
    );


/*
 * Device configuration and Device Status.
 */
typedef enum
{
    DivaDCT_SwitchType = 1,
    DivaDCT_PBXName,
    DivaDCT_DDIEnabled,
    DivaDCT_Layer2Mode,
    DivaDCT_NTMode,
    DivaDCT_NumberCollectLength,
    DivaDCT_R2Variant,

    DivaDCT_AutoSpid = 100,
    DivaDCT_SPID1,
    DivaDCT_SPID2,
    DivaDCT_DirectoryNumber1,
    DivaDCT_DirectoryNumber2,
} DivaDeviceConfigType;

typedef enum
{
    DivaSwitchType1TR6          = 0,
    DivaSwitchTypeETSI          = 1,
    DivaSwitchTypeFranceVN4     = 2,
    DivaSwitchTypeBelgium       = 3,
    DivaSwitchTypeSweden        = 4,
    DivaSwitchTypeNI_DMS        = 5,
    DivaSwitchType5ESSCustom    = 6,
    DivaSwitchTypeJapan         = 7,
    DivaSwitchTypeItaly         = 10,
    DivaSwitchTypeTaiwan        = 11,
    DivaSwitchTypeAustralia     = 12,
    DivaSwitchType4ESS_SDN      = 13,
    DivaSwitchType4ESS_SDS      = 14,
    DivaSwitchType4ESS_LDS      = 15,
    DivaSwitchType4ESS_MGC      = 16,
    DivaSwitchType4ESS_MGI      = 17,
    DivaSwitchTypeHongkong      = 18,
    DivaSwitchTypeRBSCAS        = 19,
    DivaSwitchTypeQSIG          = 21,
    DivaSwitchTypeNI_EWSD       = 22,
    DivaSwitchTypeNI_5ESS       = 23,
    DivaSwitchTypeQSIG_T1       = 27,
    DivaSwitchTypeTrunkE1       = 30,
    DivaSwitchTypeTrunkT1       = 31,
    DivaSwitchTypeR2CAS         = 32,
    DivaSwitchTypeFranceVN6     = 33,
    DivaSwitchTypePOTS          = 34,
    DivaSwitchTypeNewZealand    = 36,
    DivaSwitchTypeSerbia        = 37,
    DivaSwitchTypeLineSideE1    = 38,
} DivaSwitchType;

typedef enum
{
    DivaR2VariantChina          = 0,
    DivaR2VariantIndia          = 1,
    DivaR2VariantIndonesia      = 2,
    DivaR2VariantThailand       = 3,
    DivaR2VariantPhilipines     = 4,
    DivaR2VariantBrazil         = 5,
    DivaR2VariantMexico         = 6,
    DivaR2VariantKorea          = 7,
    DivaR2VariantArgentina      = 8,
    DivaR2VariantVenezuela      = 9,
    DivaR2VariantITU_T          = 100,
} DivaR2Variant;

typedef enum
{
    DivaLayer2ModePermanent,
    DivaLayer2ModeOnDemand,
    DivaLayer2ModeNoDisconnect,
} DivaLayer2Mode;

typedef union
{
    DivaSwitchType      SwitchType;
    unsigned char       PBXName[MAX_ADDR_LEN];
    BOOL                DIDEnabled;
    DivaLayer2Mode      Layer2Mode;
    BOOL                NTMode;
    DWORD               NumberCollectLength;
    BOOL                AutoSpid;
    unsigned char       SPID1[MAX_ADDR_LEN];
    unsigned char       SPID2[MAX_ADDR_LEN];
    unsigned char       DirectoryNumber1[MAX_ADDR_LEN];
    unsigned char       DirectoryNumber2[MAX_ADDR_LEN];
} DivaDeviceConfigValue;


typedef enum
{
    DivaDST_Layer1Status = 1,
    DivaDST_PotsLineStatus,
    DivaDST_Layer2Status,
    DivaDSP_LineDeviceStatus,

    DivaDST_RedAlarm = 10,
    DivaDST_BlueAlarm,
    DivaDST_YellowAlarm,

    DivaDST_ActiveInConnections = 100,
    DivaDST_ActiveOutConnections,

    DivaDST_TotalDSPs = 200,
    DivaDST_UsedDSPs,
    DivaDST_OutOfServiceDSPs,
    DivaDST_DSPStates,
} DivaDeviceStatusType;


typedef enum
{
    DivaLayer1Down = 0,
    DivaLayer1Up,
    DivaLayer1SyncLost,
    DivaLayer1Synchronized,
} DivaLayer1Status;

typedef enum
{
    DivaPotsLineDown = 0,
    DivaPotsLineHookOff,
    DivaPotsLineIdle,
    DivaPotsLineRing,
    DivaPotsLinePolarityReverse,
    DivaPotsLineToneAlertingSignal,
    DivaPotsLineEndofCall,
    DivaPotsLineFlashDetected,
    DivaPotsLineInUse,
} DivaPotsLineStatus;

typedef enum
{
    DivaLayer2Down = 0,
    DivaLayer2Up,
    DivaLayer2Closing,
    DivaLayer2Activating,
    DivaLayer2Initializing
} DivaLayer2Status;

typedef struct
{
    DWORD                   Channel;
    DivaPotsLineStatus      Status;
} DivaPotsLineState;


typedef enum  
{
    DivaDSPStateIdle,
    DivaDSPStateUsed,
    DivaDSPStateOutofService,
    DivaDSPStateUnavailable,
} DivaDSPState;


typedef struct
{
    DWORD                   Entries;
    DivaDSPState            State[32];
} DivaDSPStateArray;

typedef enum
{
    DivaLineDeviceStateEnabled = 0,
    DivaLineDeviceStateDisabled,
} DivaLineDeviceState;


typedef union
{
    DivaLayer1Status        Layer1Status;
    DivaPotsLineState       PotsLineState;
    DivaLayer2Status        Layer2Status;
    DivaLineDeviceState     LineDeviceStatus;

    BOOL                    RedAlarm;
    BOOL                    BlueAlarm;
    BOOL                    YellowAlarm;

    DWORD                   ActiveInConnections;
    DWORD                   ActiveOutConnections;

    DWORD                   TotalDSPs;
    DWORD                   UsedDSPs;
    DWORD                   OutOfServiceDSPs;
    DivaDSPStateArray       DSPStates;
} DivaDeviceStatusValue;




typedef enum
{
    DivaST_Layer1Statistics = 1,
} DivaDeviceStatisticsType;

typedef struct
{
    unsigned int            FramingErrors;   /* Only valid for PRI boards */
    unsigned int            CodeViolations;  /* Only valid for PRI boards */
    unsigned int            Crc4Errors;      /* Only valid for PRI boards */
    unsigned int            FrameSlips;      /* Only valid for PRI boards */
    unsigned int            InternalErrors;
    unsigned int            DmaOverloads;
    unsigned int            SampleOverruns;
    unsigned int            ProcessingOverloads;
    unsigned int            Load90Percent;
    unsigned int            Load80Percent;
    unsigned int            Load70Percent;
    unsigned int            Load60Percent;
    unsigned int            Load50Percent;
    unsigned int            PeakProcessingLoad;
} DivaLayer1Statistics;

typedef union
{
    DivaLayer1Statistics        Layer1Statistics;
} DivaDeviceStatisticsValue;


typedef enum
{
    DivaDSE_Layer1          = 0x00000001,
    DivaDSE_Layer2          = 0x00000002,
    DivaDSE_Alarms          = 0x00000004,
    DivaDSE_ServiceState    = 0x00000080,
} DivaDeviceStatusEvents;

DIVA_SERVER_API DWORD DivaGetLineDeviceConfiguration (
    DWORD                       LineDeviceId,
    DivaDeviceConfigType        Type,
    DivaDeviceConfigValue*      pValue,
    DWORD                       ValueSize
    );

DIVA_SERVER_API DWORD DivaGetLineDeviceStatus (  
    DWORD                       LineDeviceId,
    DivaDeviceStatusType        Type,
    DivaDeviceStatusValue*      pValue,
    DWORD                       ValueSize
    );

DIVA_SERVER_API DWORD DivaGetLineDeviceStatistics (  
    DWORD                       LineDeviceId,
    DivaDeviceStatisticsType    Type,
    DivaDeviceStatisticsValue*  pValue,
    DWORD                       ValueSize
    );

DIVA_SERVER_API DWORD DivaClearLineDeviceStatistics (  
    DWORD                       LineDeviceId,
    DivaDeviceStatisticsType    Type
    );

DIVA_SERVER_API DWORD DivaSetLineDeviceStatusEvents (
    DivaAppHandle               hApp,
    DWORD                       LineDeviceId,
    DivaDeviceStatusEvents      EventMask
    );

DIVA_SERVER_API DWORD DivaGetDeviceName (
    DWORD                       LineDevice,
    unsigned char*              pBuffer,
    DWORD                       BufferSize
    );

DIVA_SERVER_API DWORD DivaDeviceMgmtGetValue (
    DWORD                       LineDeviceId,
    char*                       pValueName,
    unsigned char*              pResultBuffer,
    DWORD                       BufferSize,
    DWORD*                      BytesRead
    );

DIVA_SERVER_API DWORD DivaDeviceMgmtSetValue (
    DWORD                       LineDeviceId,
    char*                       pValueName,
    unsigned char*              pValue,
    DWORD                       ValueSize,
    DWORD*                      BytesWritten
    );

DIVA_SERVER_API DWORD DivaDeviceMgmtExecute (
    DWORD                       LineDeviceId,
    char*                       pValueName
    );

/*
 * Set and get channel status. Only available on certain media and protocols
 */
typedef enum
{
    DivaChannelStatusUnblocked      = 0,            // Get Set
    DivaChannelStatusActiveBlocked  = 1,            // Get Set
    DivaChannelStatusPassiveBlocked = 2,            // Get
    DivaChannelStatusInUse          = 3,            // Get
} DivaChannelStatus;


DIVA_SERVER_API DWORD DivaGetChannelStatus (
    DWORD                       LineDeviceId,
    DWORD                       Channel, 
    DivaChannelStatus           *pResult
    );


DIVA_SERVER_API DWORD DivaSetChannelStatus ( 
    DWORD                       LineDeviceId,
    DWORD                       Channel, 
    DivaChannelStatus           Status
    );

DIVA_SERVER_API DWORD DivaSetAnalogHookState (
    DWORD                       LineDeviceId,
    DWORD                       Line,
    BOOL                        bOffHook
    );

DIVA_SERVER_API DWORD DivaSetServiceState ( 
    DWORD                       LineDeviceId,
    BOOL                        bEnable
    );

/*
 * Generic tone detector and generator.
 */
typedef enum
{
    DivaGenericToneGetSupportedServies,
    DivaGenericToneEnableOperation,
    DivaGenericToneDisableOperation,
} DivaGenericToneFunction;

typedef enum
{
    DivaSingleToneReportSignalNoiseRatio    = 0x00000002,
    DivaSingleToneReportEnergy              = 0x00000004,
    DivaSingleToneReportFrequency           = 0x00000008,
    DivaSingleToneReportEnergyVariation     = 0x00000010,
    DivaSingleToneReportFrequencyVariation  = 0x00000020,
    DivaSingleToneReportAll                 = 0x0000003e,
} DivaSingleToneReport;

typedef enum
{
    DivaDualToneReportSignalNoiseRatio      = 0x00000002,
    DivaDualToneReportEnergyLowTone         = 0x00000004,
    DivaDualToneReportEnergyHighTone        = 0x00000008,
    DivaDualToneReportFrequencyLowTone      = 0x00000010,
    DivaDualToneReportFrequencyHighTone     = 0x00000020,
    DivaDualToneReportAll                   = 0x0000003e,
} DivaDualToneReport;

typedef enum
{
    DivaSingleToneStart = 1,
    DivaDualToneStart,
    DivaSingleToneStop,
    DivaDualToneStop
} DivaGenericToneResultType;

typedef enum
{
    DivaGenericToneDoneSuccess          = 0,
    DivaGenericToneDoneParamsShrinked,
    DivaGenericToneDoneParamsIgnored,
    DivaGenericToneErrorInvalidParams   = 0x80,
    DivaGenericToneErrorOutOfResource,
    DivaGenericToneErrorMissingParams,
}DivaGenericToneResult;



typedef struct
{
    DivaGenericToneResultType   Type;
    DWORD                       TimeStamp;
    DivaGenericToneResult       Result;
    union
    {
        struct
        {
            short   SignalNoiseRatio;
            short   Energy;
            WORD    Frequency;
            WORD    AmplitudeVariation;
            WORD    FrequencyVariation;
        } Single;
        struct
        {
            short   SignalNoiseRatio;
            short   EnergyToneLow;
            short   EnergyToneHigh;
            WORD    FrequencyToneLow;
            WORD    FrequencyToneHigh;
        } Dual;
    } Tone;
} DivaToneDetectorResults;

typedef struct
{
    BOOL                        Confirmation;
    DWORD                       Info;
    DivaGenericToneFunction     Function;
    DWORD                       DataLength;
    BYTE                        Data[1];
} DivaGenericToneInfo;

DIVA_SERVER_API DWORD DivaGenerateSingleTone (
    DivaCallHandle              hdCall,
    DWORD                       Frequency,
    int                         Amplitude,
    DWORD                       Duration
    );

DIVA_SERVER_API DWORD DivaGenerateDualTone (
    DivaCallHandle              hdCall,
    DWORD                       FrequencyA,
    int                         AmplitudeA,
    DWORD                       FrequencyB,
    int                         AmplitudeB,
    DWORD                       Duration
    );

DIVA_SERVER_API DWORD DivaGeneratorStop (
    DivaCallHandle              hdCall
    );

DIVA_SERVER_API DWORD DivaDetectSingleTone (
    DivaCallHandle              hdCall,
    DWORD                       ReportFlags,
    DWORD                       MinDuration,
    int                         MinSNR,
    int                         MinLevel,
    DWORD                       MaxAM,
    DWORD                       MaxFM
    );

DIVA_SERVER_API DWORD DivaDetectDualTone (
    DivaCallHandle              hdCall,
    DWORD                       ReportFlags,
    DWORD                       MinDuration,
    int                         MinSNR,
    int                         MinLevel,
    int                         MaxDiffHighToLow,
    int                         MaxDiffLowToHigh
    );

DIVA_SERVER_API DWORD DivaGetToneDetectorResult (
    DivaCallHandle              hdCall,
    DivaToneDetectorResults     *pResults
    );

DIVA_SERVER_API DWORD DivaDetectorStop (
    DivaCallHandle              hdCall
    );


DIVA_SERVER_API DWORD DivaSendGenericToneRequest (
    DivaCallHandle              hdCall,
    DivaGenericToneFunction     Function,
    BYTE                        *pRequest,
    DWORD                       RequestLen,
    void                        *Handle
    );

DIVA_SERVER_API DWORD DivaEnableGenericToneReport (
    DivaCallHandle              hdCall,
    BOOL                        bEnable
    );

DIVA_SERVER_API DWORD DivaGetGenericToneInfo (
    DivaCallHandle              hdCall,
    DivaGenericToneInfo         *pInfoBuffer,
    DWORD                       InfoBufferLen,
    DWORD                       *pBytesWritten
    );

DIVA_SERVER_API DWORD DivaSpecifyCustomTone (
    DivaCallHandle              hdCall,
    DWORD                       Type,
    DWORD                       Recurrences,
    DWORD                       NumDefinitions,
    DivaToneDefinition*         pDefinitions
    );

typedef enum
{
    DivaFSKModulationV23        = 1,
} DivaFSKModulation;

typedef enum
{
    DivaFSKEventData            = 0,
    DivaFSKEventCarrierOn       = 14,
    DivaFSKEventCarrierOff      = 15,
    DivaFSKEventPreamble        = 200,
} DivaFSKEventTypes;

typedef enum
{
    FSKDetectorOptionDefault     = 0x0000,
    FSKDetectorOptionTransparent = 0x0001,
} FSKDetectorOptions;

DIVA_SERVER_API DWORD DivaDetectFSKData (
    DivaCallHandle              hCall,
    int                         Sensitivity,
    DivaFSKModulation           Modulation,
    DWORD                       Options
    );

DIVA_SERVER_API DWORD DivaStopDetectFSKData (
    DivaCallHandle              hCall
    );

/*
 * Timer services for global timer of an application of per call.
 */
DIVA_SERVER_API DWORD DivaStartApplicationTimer (
    DivaAppHandle               Handle,
    DWORD                       WaitTime
    );

DIVA_SERVER_API DWORD DivaStopApplicationTimer (
    DivaAppHandle               Handle
    );

DIVA_SERVER_API DWORD DivaStartCallTimer (
    DivaCallHandle              hdCall,
    DWORD                       WaitTime
    );

DIVA_SERVER_API DWORD DivaStopCallTimer (
    DivaCallHandle              hdCall
    );



/*
 * IP specific functions to specify the codecs to be negotiated.
 */
typedef enum
{
    DivaCodec_G711_ALaw = 1,
    DivaCodec_G711_ULaw,
    DivaCodec_T38,
    DivaCodec_G723,
    DivaCodec_G723_5_3,
    DivaCodec_G723_6_3,
    DivaCodec_G726,
    DivaCodec_G726_16,
    DivaCodec_G726_24,
    DivaCodec_G726_32,
    DivaCodec_G726_40,
    DivaCodec_G729,
    DivaCodec_G729A,
    DivaCodec_G729B,
    DivaCodec_G729AB,
    DivaCodec_GSM_FULL,
    DivaCodec_GSM_HALF,
    DivaCodec_GSM_EFR,
    DivaCodec_GSM_AMR,
} DivaCodec;

typedef enum
{
    DivaDtmfModeAuto = 0,
    DivaDtmfModeRFC2833,
    DivaDtmfModeSIPInfo,
    DivaDtmfModeRFC2833SIPInfo,
} DivaDTMFMode;


DIVA_SERVER_API DWORD   DivaResetDefaultCodecList (
    DWORD           LineDevice
    );

DIVA_SERVER_API DWORD   DivaAddDefaultCodec (
    DWORD           LineDevice,
    DivaCodec       Codec,
    DWORD           FramesOrSize,
    DivaDirection   Direction
    );

DIVA_SERVER_API DWORD   DivaResetCodecList (
    DivaCallHandle  hdCall
    );

DIVA_SERVER_API DWORD   DivaAddCodec (
    DivaCallHandle  hdCall,
    DivaCodec       Codec,
    DWORD           FramesOrSize,
    DivaDirection   Direction
    );

/*
 * SIP specific funtions.
 */
DIVA_SERVER_API DWORD   DivaRegisterSIPHeader (
    DWORD           LineDevice,
    const char*     pName
    );

typedef enum
{
    DivaSIPInviteMessageHandle = 1,
} DivaSIPMessageHandle;

DIVA_SERVER_API DWORD   DivaGetSIPHeader (
    DivaCallHandle  hdCall,
    DivaHandle      hMessage,
    char*           pName,
    char*           pBuffer,
    DWORD           BufferSize,
    DWORD*          pSizeNeeded
    );

DIVA_SERVER_API DWORD   DivaSetSIPHeader (
    DivaCallHandle  hdCall,
    const char*     pName,
    const char*     pHeader
    );

typedef enum
{
    DivaCodecMaskG711aLaw       = 0x00000001,
    DivaCodecMaskG711uLaw       = 0x00000002,
    DivaCodecMaskDynamicDTMF    = 0x00000004,
    DivaCodecMaskClearMode      = 0x00100000,
} DivaCodecMask;

typedef enum
{
    DivaMediaChannelUp = 1,
    DivaMediaChannelDown,
    DivaMediaChannelNoResources,
    DivaMediaChannelError,
} DivaMediaChannelStatus;

DIVA_SERVER_API DWORD   DivaCreateIPMediaChannel ( 
    DivaAppHandle   hApp,
    AppCallHandle   haCall,
    DivaCallHandle  *phdCall
    );

DIVA_SERVER_API DWORD   DivaConnectIPMediaChannel ( 
    DivaCallHandle  hdCall,
    char*           LocalIpAddress,
    DWORD           LocalPort,
    char*           RemoteIpAddress,
    DWORD           RemotePort,
    DWORD           CodecMask,
    DWORD           Options
    );

DIVA_SERVER_API DWORD   DivaDisconnectIPMediaChannel ( 
    DivaCallHandle  hdCall
    );

DIVA_SERVER_API DWORD   DivaCloseIPMediaChannel ( 
    DivaCallHandle  hdCall
    );


/*
 * Speech integration via MRCP.
 */

typedef enum
{
    DivaMrcpVersion1 = 1,
    DivaMrcpVersion2,
} DivaMrcpVersion;

typedef enum
{
    DivaSpeechRecognizerStatusClosed = 0,
    DivaSpeechRecognizerStatusOpening,
    DivaSpeechRecognizerStatusOpened,
    DivaSpeechRecognizerStatusClosing,
    DivaSpeechRecognizerStatusOpenFailed,
} DivaSpeechRecognizerStatus; 

typedef enum
{
    DivaSpeechRecognitionInProgress = 1,
    DivaSpeechRecognitionStartOfSpeech,
    DivaSpeechRecognitionCompleted,
    DivaSpeechRecognitionUserTerminated,
    DivaSpeechRecognitionNoMatch,
    DivaSpeechRecognitionNoInputTimeout,
    DivaSpeechRecognitionTimeout,
    DivaSpeechRecognitionGrammarError,
    DivaSpeechRecognitionGeneralError,
    DivaSpeechRecognitionSpeechTooEarly,
    DivaSpeechRecognitionSpeechTooLong,
    DivaSpeechRecognitionUnsupportedLanguage,
} DivaSpeechRecognitionProgress;

typedef enum
{
    DivaSpeechRecognizerResultInterpretation = 1,
    DivaSpeechRecognizerResultGrammar,
    DivaSpeechRecognizerResultContent
} DivaSpeechRecognizerResultType;


DIVA_SERVER_API DWORD   DivaInitializeSpeechProcessing (
    const char*                         LocalIpAddress
    );

DIVA_SERVER_API DWORD   DivaOpenSpeechRecognizer (
    DivaCallHandle                      hdCall,
    const char*                         RecognizerName
    );

DIVA_SERVER_API DWORD   DivaCloseSpeechRecognizer (
    DivaCallHandle                      hdCall
    );

DIVA_SERVER_API DWORD   DivaStartSpeechRecognizer (
    DivaCallHandle                      hdCall,
    BOOL                                bAutoRestart
    );

DIVA_SERVER_API DWORD   DivaStopSpeechRecognizer (
    DivaCallHandle                      hdCall
    );

DIVA_SERVER_API DWORD   DivaGetSpeechRecognizerResult (
    DivaCallHandle                      hdCall,
    char*                               pTextBuffer,
    DWORD*                              pTextSize,
    DWORD*                              pConfidence
    );

DIVA_SERVER_API DWORD   DivaGetSpeechRecognizerResultDetails (
    DivaCallHandle                      hdCall,
    DivaSpeechRecognizerResultType      Type,
    char*                               pBuffer,
    DWORD*                              pSize
    );

DIVA_SERVER_API DWORD   DivaSetSpeechRecognizerParameter (
    DivaCallHandle                      hdCall,
    const char*                         Name,
    const char*                         Value
    );

DIVA_SERVER_API DWORD   DivaSetSpeechRecognizerGrammar (
    DivaCallHandle                      hdCall,
    const char*                         ContentType,
    const char*                         ContentId,
    const char*                         Grammar
    );

DIVA_SERVER_API DWORD   DivaCreateSpeechRecognizer (
    const char*                         Name,
    DivaMrcpVersion                     Version,
    const char*                         ServerName,
    const char*                         ServerIpAddress,
    DWORD                               ServerPort,
    const char*                         DefaultMediaPath,
    const char*                         LocalIpAddress
    );

DIVA_SERVER_API DWORD   DivaSetSpeechRecognizerDefaultParameter (
    const char*                         Name,
    const char*                         ParameterName,
    const char*                         ParameterValue
    );

DIVA_SERVER_API DWORD   DivaSetSpeechRecognizerDefaultGrammar (
    const char*                         Name,
    const char*                         ContentType,
    const char*                         ContentId,
    const char*                         Grammar );

#ifdef __cplusplus
}
#endif

#endif /* _DIVA_SERVER_SDK_H_ */

